/*
  *** Blocktrap GBA ***
 Franois PESSAUX September 2004
 This program is freeware.
 Use its sources as you whant provided that you leave this copyright unchanged
 and that you drop a small word telling that I'm the author of these sources.
 Thank's a lot.
   --Francois
*/

#include <string.h>
#include <stdlib.h>
#include <gba.h>
#include <screen_mode.h>
#include <interrupt.h>
#include <timer.h>
#include "game.h"
#include "init.h"
#include "ints.h"
#include "pause.h"
#include "control.h"
#include "print.h"
#include "background.h"



vu16 key_prev = 0xFFFF ;
vu16 key_curr = 0xFFFF ;





block make_random_new_block (struct game *game)
{
  enum block_kind new_block_kind ;
  enum color new_block_color = Indep ;
  short proba ;

  proba = (short) (((unsigned short) rand ()) % 100) ;
  if (proba <= game->percent_chances_of_bonus) {
    /* Gnration d'un bloc spcial. */
    new_block_kind =
      (enum block_kind)
      ((((unsigned short) rand ()) % (NB_BLOCK_KINDS - NB_NON_SPECIAL_KINDS)) +
       NB_NON_SPECIAL_KINDS) ;
    /* Dtermination d'une ventuelle couleur pour le bloc spcial. */
    switch (new_block_kind) {
    case Block_Autofire:
    case Block_Deep_charge:
    case Block_Random_Direction:
    case Block_Inverted_Movement:
    case Block_Color_Cleaner:
    case Block_Painter:
    case Block_Line_Col_Cleaner:
    case Block_Multidirection:
    case Block_Speed_Increase:
    case Block_Speed_Decrease:
    case Block_Antidote:
    case Block_Eraser:
      new_block_color = (enum color) (((unsigned short) rand ()) % NB_COLORS) ;
      break ;

    case Block_Screen_Cleaner:
    case Block_Random_Color:
    case Block_Color_Mixer:
    case Block_Furry_Moving:
    case Block_Furry_Spinning:
    case Block_Blind:
      new_block_color = Indep ;
      break ;

    default:
      notify_long_message (FALSE, "make_random_new_block\nASSERT 0") ;
      break ;
    }
  }
  else {
    /* Gnration d'un bloc normal. */
    new_block_kind = Block_Regular ;
    new_block_color = (enum color) (((unsigned short) rand ()) % NB_COLORS) ;
  }
  return (MAKE_BLOCK (new_block_kind, new_block_color)) ;
}





void game_main_loop (struct game *game)
{
  block new_block ;
  unsigned long previous_insertion_time, instant_elapsed_ticks ;

  elapsed_ticks = 0 ;
  instant_elapsed_ticks = 0 ;
  previous_insertion_time = 0 ;
  RESET_GAME_TIMER ;
  TURN_ON_GAME_TIMER ;

  while (game->game_running) {
    instant_elapsed_ticks = elapsed_ticks ;
    /* Gestion des altrations de l'tat du joueur. */
    if (game->player.status.status & Status_Inverted_Movement) {
      if (instant_elapsed_ticks >=
	  game->player.status.inverted_movement.end) {
	game->player.status.inverted_movement.end = 0L ;
	game->player.status.inverted_movement.previous_action = 0L ;
	game->player.status.status &= ~(Status_Inverted_Movement) ;
      }
      /* Rien d'autre  faire sinon, car gr plus bas lors */
      /* de la prise en compte des ordres du joueur.        */
    }
    if (game->player.status.status & Status_Blind) {
      if (instant_elapsed_ticks >= game->player.status.blind.end) {
	game->player.status.blind.end = 0L ;
	game->player.status.blind.previous_action = 0L ;
	game->player.status.status &= ~(Status_Blind) ;
	set_blending_mode (0) ;
      }
      else {
	if (instant_elapsed_ticks >
	    game->player.status.blind.previous_action + 3) {
	  while (REG_VCOUNT < 160) ;
	  /* 4 chances sur 5 d'avoir un cran noir. */
	  REG_COLEY = (u16) (((rand ()) % 5) < 4 ? 15 : 0) ;
	  set_blending_mode (FADE_MODE_3 | FADE_SOURCE_BG0 | FADE_SOURCE_BG1) ;
	}
      }
    }
    /* Gestion des dplacements incontrls. */
    if (game->player.status.status & Status_Furry_Moving) {
      if (instant_elapsed_ticks >= game->player.status.furry_moving.end) {
	game->player.status.furry_moving.end = 0L ;
	game->player.status.furry_moving.previous_action = 0L ;
	game->player.status.status &= ~(Status_Furry_Moving) ;
      }
      else {
	if (instant_elapsed_ticks >
	    game->player.status.furry_moving.previous_action) {
	  analyse_player_input (game, ((enum action) (rand () % 4))) ;
	  game->player.status.furry_moving.previous_action =
	    instant_elapsed_ticks ;
	}
      }
    }
    /* Gestion des rotations incontrles. */
    if (game->player.status.status & Status_Furry_Spinning) {
      if (instant_elapsed_ticks >= game->player.status.furry_spinning.end) {
	game->player.status.furry_spinning.end = 0L ;
	game->player.status.furry_spinning.previous_action = 0L ;
	game->player.status.status &= ~(Status_Furry_Spinning) ;
      }
      else {
	if (instant_elapsed_ticks >
	    game->player.status.furry_spinning.previous_action) {
	  analyse_player_input
	    (game,
	     ((enum action) (Turn_clockwise + (rand () % 2)))) ;
	  game->player.status.furry_spinning.previous_action =
	    instant_elapsed_ticks ;
	}
      }
    }
    /* Gestion des tirs incontrls. */
    if (game->player.status.status & Status_Auto_Fire) {
      if (instant_elapsed_ticks >= game->player.status.auto_fire.end) {
	game->player.status.auto_fire.end = 0L ;
	game->player.status.auto_fire.previous_action = 0L ;
	game->player.status.status &= ~(Status_Auto_Fire) ;
      }
      else {
	/* On tire tous les 2 ticks. */
	if (instant_elapsed_ticks >
	    game->player.status.auto_fire.previous_action + 2) {
	  analyse_player_input (game, Fire) ;
	  game->player.status.auto_fire.previous_action =
	    instant_elapsed_ticks ;
	}
      }
    }

    /* Gestion de l'acclration de la vitesse d'insertion. */
    if (game->player.status.status & Status_Speed_Increased) {
      if (instant_elapsed_ticks >= game->player.status.speed_increased.end) {
	game->player.status.speed_increased.end = 0L ;
	game->player.status.speed_increased.previous_action = 0L ;
	game->player.status.status &= ~(Status_Speed_Increased) ;
	/* On restaure la temporisation normale. */
	game->temporisation = get_tempo_from_level (game->level) ;
      }
    }

    /* Gestion de la rduction de la vitesse d'insertion. */
    if (game->player.status.status & Status_Speed_Decreased) {
      if (instant_elapsed_ticks >= game->player.status.speed_decreased.end) {
	game->player.status.speed_decreased.end = 0L ;
	game->player.status.speed_decreased.previous_action = 0L ;
	game->player.status.status &= ~(Status_Speed_Decreased) ;
	/* On restaure la temporisation normale. */
	game->temporisation = get_tempo_from_level (game->level) ;
      }
    }

    /* Acquisition des touches. */
    KEY_POOL ;
    if (KEY_HIT (KEYPAD_LEFT))
      analyse_player_input
	(game,
	 (game->player.status.status & Status_Inverted_Movement ?
	  Go_right : Go_left)) ;
    else
      if (KEY_HIT (KEYPAD_RIGHT))
	analyse_player_input
	  (game,
	   (game->player.status.status & Status_Inverted_Movement ?
	    Go_left : Go_right)) ;
      else
	if (KEY_HIT (KEYPAD_UP))
	  analyse_player_input
	    (game,
	     (game->player.status.status & Status_Inverted_Movement ?
	      Go_down : Go_up)) ;
	else
	  if (KEY_HIT (KEYPAD_DOWN))
	    analyse_player_input
	      (game,
	       (game->player.status.status & Status_Inverted_Movement ?
		Go_up : Go_down)) ;
	  else
	    if (KEY_HIT (BUTTON_A)) analyse_player_input (game, Fire) ;
	    else
	      if (KEY_HIT (BUTTON_B))
		analyse_player_input (game, Invoke_bonus) ;
	      else
		if (KEY_HIT (BUTTON_LEFT))
		  analyse_player_input
		    (game,
		     (game->player.status.status & Status_Inverted_Movement ?
		      Turn_clockwise : Turn_reverse)) ;
		else
		  if (KEY_HIT (BUTTON_RIGHT))
		    analyse_player_input
		      (game,
		       (game->player.status.status & Status_Inverted_Movement ?
			Turn_reverse : Turn_clockwise)) ;
		  else
		    if (KEY_HIT (BUTTON_START)) show_pause () ;

    if (previous_insertion_time + game->temporisation <=
	instant_elapsed_ticks) {
      new_block = make_random_new_block (game) ;
      /* On rajoute un bloc. */
      add_block_and_check_end_of_game
	(game,
	 ((unsigned short) rand ()) % SQUARE_SIZE,
	 (enum direction) (((unsigned short) rand ()) % 3),
	 new_block) ;
      previous_insertion_time = instant_elapsed_ticks ;
    }
  }

  /* On arrte le timer. */
  TURN_OFF_GAME_TIMER ;
}






void handle_reserve_input (struct game *game)
{
  u8 cursor_selection, old_cursor_selection ;
  
  cursor_selection = SELECTION_Deep_Charge_Red ;

  for (;;) {
    old_cursor_selection = cursor_selection ;
    KEY_POOL ;

    /* Le bouton B fait sortir du mode de slection de bonus en rserve. */
    if (KEY_HIT (BUTTON_B)) return ;

    /* Le bouton A est le bouton d'activation s'il y a un bonus dispo. */
    if (KEY_HIT (BUTTON_A)) {
      switch (cursor_selection) {
      case SELECTION_Deep_Charge_Red:
	if (game->reserve[BONUS_Deep_Charge_Red_INDEX] > 0) {
	  game->reserve[BONUS_Deep_Charge_Red_INDEX]-- ;
	  game->player.bullet = Bullet_Deep_Charge ;
	  game->player.color = Red ;
	  return ;
	}
	break ;
      case SELECTION_Deep_Charge_Blue:
	if (game->reserve[BONUS_Deep_Charge_Blue_INDEX] > 0) {
	  game->reserve[BONUS_Deep_Charge_Blue_INDEX]-- ;
	  game->player.bullet = Bullet_Deep_Charge ;
	  game->player.color = Blue ;
	  return ;
	}
	break ;
      case SELECTION_Deep_Charge_Green:
	if (game->reserve[BONUS_Deep_Charge_Green_INDEX] > 0) {
	  game->reserve[BONUS_Deep_Charge_Green_INDEX]-- ;
	  game->player.bullet = Bullet_Deep_Charge ;
	  game->player.color = Green ;
	  return ;
	}
	break ;
      case SELECTION_Deep_Charge_Yellow:
	if (game->reserve[BONUS_Deep_Charge_Yellow_INDEX] > 0) {
	  game->reserve[BONUS_Deep_Charge_Yellow_INDEX]-- ;
	  game->player.bullet = Bullet_Deep_Charge ;
	  game->player.color = Yellow ;
	  return ;
	}
	break ;
      case SELECTION_Painter_Red:
	if (game->reserve[BONUS_Painter_Red_INDEX] > 0) {
	  game->reserve[BONUS_Painter_Red_INDEX]-- ;
	  game->player.bullet = Bullet_Painter ;
	  game->player.color = Red ;
	  return ;
	}
	break ;
      case SELECTION_Painter_Blue:
	if (game->reserve[BONUS_Painter_Blue_INDEX] > 0) {
	  game->reserve[BONUS_Painter_Blue_INDEX]-- ;
	  game->player.bullet = Bullet_Painter ;
	  game->player.color = Blue ;
	  return ;
	}
	break ;
      case SELECTION_Painter_Green:
	if (game->reserve[BONUS_Painter_Green_INDEX] > 0) {
	  game->reserve[BONUS_Painter_Green_INDEX]-- ;
	  game->player.bullet = Bullet_Painter ;
	  game->player.color = Green ;
	  return ;
	}
	break ;
      case SELECTION_Painter_Yellow:
	if (game->reserve[BONUS_Painter_Yellow_INDEX] > 0) {
	  game->reserve[BONUS_Painter_Yellow_INDEX]-- ;
	  game->player.bullet = Bullet_Painter ;
	  game->player.color = Yellow ;
	  return ;
	}
	break ;
      case SELECTION_Line_Col_Cleaner:
	if (game->reserve[BONUS_Line_Col_Cleaner_INDEX] > 0) {
	  game->reserve[BONUS_Line_Col_Cleaner_INDEX]-- ;
	  game->player.bullet = Bullet_Line_Col_Cleaner ;
	  game->player.color = Indep ;
	  return ;
	}
	break ;
      case SELECTION_Antidote:
	if (game->reserve[BONUS_Antidote_INDEX] > 0) {
	  game->reserve[BONUS_Antidote_INDEX]-- ;
	  current_game.player.status.status = Status_Normal ;
	  return ;
	}
	break ;
      case SELECTION_Eraser:
	if (game->reserve[BONUS_Eraser_INDEX] > 0) {
	  game->reserve[BONUS_Eraser_INDEX]-- ;
	  game->player.bullet = Bullet_Eraser ;
	  game->player.color = Indep ;
	  return ;
	}
	break ;
      default:
	notify_long_message (FALSE, "handle_reserve_input\nASSERT 0") ;
	break ;
      }
    }

    if (KEY_HIT (KEYPAD_UP)) {
      /* Dplacement vers le haut. */
      switch (cursor_selection) {
      case SELECTION_Deep_Charge_Red:
	cursor_selection = SELECTION_Deep_Charge_Yellow ;
	break ;
      case SELECTION_Deep_Charge_Blue:
      case SELECTION_Deep_Charge_Green:
      case SELECTION_Deep_Charge_Yellow:
	cursor_selection-- ;
	break ;
      case SELECTION_Painter_Red:
	cursor_selection = SELECTION_Painter_Yellow ;
	break ;
      case SELECTION_Painter_Blue:
      case SELECTION_Painter_Green:
      case SELECTION_Painter_Yellow:
	cursor_selection-- ;
	break ;
      case SELECTION_Line_Col_Cleaner:
      case SELECTION_Antidote:
      case SELECTION_Eraser:
	break ;
      default:
	notify_long_message (FALSE, "handle_reserve_input\nASSERT 1") ;
	break ;
      }
    }
    
    if (KEY_HIT (KEYPAD_DOWN)) {
      /* Dplacement vers le bas. */
      switch (cursor_selection) {
      case SELECTION_Deep_Charge_Red:
      case SELECTION_Deep_Charge_Blue:
      case SELECTION_Deep_Charge_Green:
	cursor_selection++ ;
	break ;
      case SELECTION_Deep_Charge_Yellow:
	cursor_selection = SELECTION_Deep_Charge_Red ;
	break ;
      case SELECTION_Painter_Red:
      case SELECTION_Painter_Blue:
      case SELECTION_Painter_Green:
	cursor_selection++ ;
	break ;
      case SELECTION_Painter_Yellow:
	cursor_selection = SELECTION_Painter_Red ;
	break ;
      case SELECTION_Line_Col_Cleaner:
      case SELECTION_Antidote:
      case SELECTION_Eraser:
	break ;
      default:
	notify_long_message (FALSE, "handle_reserve_input\nASSERT 2") ;
	break ;
      }
    }

    if (KEY_HIT (KEYPAD_LEFT)) {
      /* Dplacement vers la gauche. */
      switch (cursor_selection) {
      case SELECTION_Deep_Charge_Red:
      case SELECTION_Deep_Charge_Blue:
      case SELECTION_Deep_Charge_Green:
      case SELECTION_Deep_Charge_Yellow:
	cursor_selection = SELECTION_Eraser ;
	break ;
      case SELECTION_Painter_Red:
      case SELECTION_Painter_Blue:
      case SELECTION_Painter_Green:
      case SELECTION_Painter_Yellow:
	cursor_selection -= 4 ;
	break ;
      case SELECTION_Line_Col_Cleaner:
	cursor_selection = SELECTION_Painter_Red ;
	break ;
      case SELECTION_Antidote:
      case SELECTION_Eraser:
	cursor_selection-- ;
	break ;
      default:
	notify_long_message (FALSE, "handle_reserve_input\nASSERT 3") ;
	break ;
      }
    }
    
    if (KEY_HIT (KEYPAD_RIGHT)) {
      /* Dplacement vers la droite. */
      switch (cursor_selection) {
      case SELECTION_Deep_Charge_Red:
      case SELECTION_Deep_Charge_Blue:
      case SELECTION_Deep_Charge_Green:
      case SELECTION_Deep_Charge_Yellow:
	cursor_selection += 4 ;
	break ;
      case SELECTION_Painter_Red:
      case SELECTION_Painter_Blue:
      case SELECTION_Painter_Green:
      case SELECTION_Painter_Yellow:
	cursor_selection = SELECTION_Line_Col_Cleaner ;
	break ;
      case SELECTION_Line_Col_Cleaner:
      case SELECTION_Antidote:
	cursor_selection++ ;
	break ;
      case SELECTION_Eraser:
	cursor_selection = SELECTION_Deep_Charge_Red ;
	break ;
      default:
	notify_long_message (FALSE, "handle_reserve_input\nASSERT 4") ;
	break ;
      }
    }

    /* On change la position du curseur seulement s'il a boug. */
    if (cursor_selection != old_cursor_selection) {
      u16 x = BONUS_Deep_Charge_AREA_X ;
      u16 y = BONUS_Deep_Charge_Red_AREA_Y ;

      switch (cursor_selection) {
      case SELECTION_Deep_Charge_Red:
	x = BONUS_Deep_Charge_AREA_X ;
	y = BONUS_Deep_Charge_Red_AREA_Y  ;
	break ;
      case SELECTION_Deep_Charge_Blue:
	x = BONUS_Deep_Charge_AREA_X ;
	y = BONUS_Deep_Charge_Blue_AREA_Y ;
	break ;
      case SELECTION_Deep_Charge_Green:
	x = BONUS_Deep_Charge_AREA_X ;
	y = BONUS_Deep_Charge_Green_AREA_Y ;
	break ;
      case SELECTION_Deep_Charge_Yellow:
	x = BONUS_Deep_Charge_AREA_X ;
	y = BONUS_Deep_Charge_Yellow_AREA_Y ;
	break ;
      case SELECTION_Painter_Red:
	x = BONUS_Painter_AREA_X ;
	y = BONUS_Painter_Red_AREA_Y ;
	break ;
      case SELECTION_Painter_Blue:
	x = BONUS_Painter_AREA_X ;
	y = BONUS_Painter_Blue_AREA_Y ;
	break ;
      case SELECTION_Painter_Green:
	x = BONUS_Painter_AREA_X ;
	y = BONUS_Painter_Green_AREA_Y ;
	break ;
      case SELECTION_Painter_Yellow:
	x = BONUS_Painter_AREA_X ;
	y = BONUS_Painter_Yellow_AREA_Y ;
	break ;
      case SELECTION_Line_Col_Cleaner:
	x = BONUS_Line_Col_Cleaner_AREA_X ;
	y = BONUS_Line_Col_Cleaner_AREA_Y ;
	break ;
      case SELECTION_Antidote:
	x = BONUS_Antidote_AREA_X ;
	y = BONUS_Antidote_AREA_Y ;
	break ;
      case SELECTION_Eraser :
	x = BONUS_Eraser_AREA_X ;
	y = BONUS_Eraser_AREA_Y ;
	break ;
      default:
	notify_long_message (FALSE, "handle_reserve_input\nASSERT 5") ;
	break ;
      }

      cursor_sprite.attribute1 = cursor_sprite.attribute1 & 0xFE00 ;
      /* + 8 pour corriger vis--vis de la forme du sprite. */
      cursor_sprite.attribute1 = cursor_sprite.attribute1 | ((x * 8) + 8) ;
      cursor_sprite.attribute0 = cursor_sprite.attribute0 & 0xFF00 ;
      cursor_sprite.attribute0 = cursor_sprite.attribute0 | ((y * 8) - 8) ;
      /* Transfert des infos de sprite dans la mmoire vido-sprite. */
      while (REG_VCOUNT < 160) ;
      memcpy (sprite_video_memory, &cursor_sprite, sizeof (struct sprite)) ;
    }
  }
}





void reserve_selection (struct game *game)
{
  u16 old_screen_mode ;

  /* On sauvegarde le mode vido d'origine. */
  old_screen_mode = REG_DISPCNT ;

  /* On active le sprite de curseur. */
  /* Efface l'ancienne abscisse. */
  cursor_sprite.attribute1 = cursor_sprite.attribute1 & 0xFE00 ;
  /* + 8 pour corriger vis--vis de la forme du sprite. */
  cursor_sprite.attribute1 =
    cursor_sprite.attribute1 | ((BONUS_Deep_Charge_AREA_X * 8) + 8) ;
  /* Efface l'ancienne ordonne. */
  cursor_sprite.attribute0 = cursor_sprite.attribute0 & 0xFF00 ;
  /* Mme genre de correction pour l'ordonne. */
  cursor_sprite.attribute0 =
    cursor_sprite.attribute0 | ((BONUS_Deep_Charge_Red_AREA_Y * 8) - 8) ;
  /* Transfert des infos de sprite dans la mmoire vido-sprite. */
  while (REG_VCOUNT < 160) ;
  memcpy (sprite_video_memory, &cursor_sprite, sizeof (struct sprite)) ;
  set_screen_mode (old_screen_mode | OBJ_MAP_1D | OBJ_ENABLE) ;

  handle_reserve_input (game) ;

  /* On restaure le mode vido d'origine. */
  set_screen_mode (old_screen_mode) ;
}
