/*
  *** Blocktrap GBA ***
 Franois PESSAUX September 2004
 This program is freeware.
 Use its sources as you whant provided that you leave this copyright unchanged
 and that you drop a small word telling that I'm the author of these sources.
 Thank's a lot.
   --Francois
*/

#include <string.h>
#include <gba.h>
#include <screen_mode.h>
#include <interrupt.h>
#include "game.h"
#include "control.h"
#include "fame_gfx.h"
#include "fame.h"
#include "persistent.h"
#include "print.h"
#include "blocks.h"
#include "trigtbl.h"
#include "init.h"
#include "ints.h"
#include "ko.h"
#include "win.h"
#include "wav_end.h"



void mosaic_transition_bg2 ()
{
  u8 i ;

  REG_BG2CNT = REG_BG2CNT | BACKGROUND_MOSAIC_ENABLED ;
  for (i = 0; i < 16; i++) {
    while (REG_VCOUNT < 160) ;
    set_mosaic_mode (i, i, 0, 0) ;
  }

  /* On supprime l'effet de mosaique. */
  REG_BG2CNT = REG_BG2CNT & (~BACKGROUND_MOSAIC_ENABLED) ;
}




void print_fame_scores ()
{
  short column ;
  unsigned int div ;
  unsigned int remain = 0 ;
  u8 rang ;
  u16 tile_on_screen ;
  u16 *fame_video_memory = SCREEN_BASE_BLOCK_11_ADDR ;

  for (rang = 0; rang < 5; rang++) {
    div = current_hall_of_fame[rang] ;

    /* Affichage sur 10 chiffres (2^32 = 4294967296). */
    for (column = 9; column >= 0; column --) {
      remain = div % 10 ;
      div = div / 10 ;
      tile_on_screen =
	((SCORE_START_Y_AREA + (rang * SCORE_VSPACING)) * (256 / 8)) +
	(SCORE_START_X_AREA + column) ;
      /* Premire tile du chiffre. */
      fame_video_memory[tile_on_screen] =
	(u16) ((FAME_DIGITS_TILES_START_INDEX  + remain) |
	       (MISC_REQUESTERS_PALETTE_SLOT << 12)) ;
      /* Seconde tile du chiffre. */
      tile_on_screen =
	(((SCORE_START_Y_AREA + (rang * SCORE_VSPACING)) + 1) * (256 / 8)) +
	(SCORE_START_X_AREA + column) ;
      fame_video_memory[tile_on_screen] =
	(u16) ((FAME_DIGITS_TILES_START_INDEX  + remain +
		(NB_FAME_DIGITS_TILES / 2)) | (MISC_REQUESTERS_PALETTE_SLOT << 12)) ;
    }
  }
}



void show_hall_fame ()
{
  u16 *palette_memory = BACKGROUND_PALETTE_ADDR ;
  u16 *fame_video_memory = SCREEN_BASE_BLOCK_11_ADDR ;
  u16 *tiles_memory = CHAR_BASE_BLOCK_2_ADDR ;
  u16 old_screen_mode ;
  u16 tile_index,  map_index ;
  u16 i, j ;

  /* On sauvegarde le mode vido d'origine. */
  old_screen_mode = REG_DISPCNT ;
  set_screen_mode (0) ;

  /* Layer de requesters divers de saisie. */
  set_background2 (BACKGROUND_PRIORITY_1 |
		   BACKGROUND_CHAR_BASE_BLOCK_2 |
		   BACKGROUND_COLOR_16 |
		   BACKGROUND_SCREEN_BASE_BLOCK_11 |
		   TEXT_BACKGROUND_256x256) ;

  tile_index = 0 ;
  /* Chargement des tiles de fond en mmoire. */
  for (i = 0; i < NB_FAME_TILES * (32 / 2); i++) {
    tiles_memory[tile_index] =
      (u16) ((fame_tiles[(i * 2) + 1] << 8) |
	     (fame_tiles[(i * 2)])) ;
    tile_index++ ;
  }

  /* Chargement des tiles de chiffres de score en mmoire. */
  for (i = 0; i < NB_FAME_DIGITS_TILES * (32 / 2); i++) {
    tiles_memory[tile_index] =
      (u16) ((fame_digit_tiles[(i * 2) + 1] << 8) |
	     (fame_digit_tiles[(i * 2)])) ;
    tile_index++ ;
  }

  /* Chargement de la map de fond. */
  map_index = 0 ;
  for (j=0; j<20; j++) {          /* Boucle verticale. */
    for (i=0; i<30; i++) {        /* Boucle honrizontale. */
      /* Terrain de jeu. */
      fame_video_memory[(j * (256 / 8)) + i] =
	(u16) ((fame_map[map_index]) | (MISC_REQUESTERS_PALETTE_SLOT  << 12)) ;
      map_index++ ;
    }
  }

  /* Chargement de la palette de fond en slot 0. */
  i = MISC_REQUESTERS_PALETTE_SLOT * 16 ;
  for (j = 0; j < 16; i++, j++)
    palette_memory[i] = fame_palette[j] ;

  /* Positionnement des offsets de scrolling en haut  gauche (0, 0). */
  REG_BG2HOFS = 0 ;
  REG_BG2VOFS = 0 ;

  print_fame_scores () ;
  set_screen_mode (MODE_0 | BG2_ENABLE) ;

  for (;;) {
    /* On attend une pression sur le bouton A, B */
    /* ou SELECT pour faire disparatre le tout. */
    KEY_POOL ;
    if ((KEY_HIT (BUTTON_A)) || (KEY_HIT (BUTTON_B)) ||
	(KEY_HIT (BUTTON_START))) break ;
  }

  /* Petite anim de transition... */
  mosaic_transition_bg2 () ;

  /* On restaure le mode vido d'origine. */
  set_screen_mode (old_screen_mode) ;
}




void rotate_background2 (int angle, int center_x, int center_y,
			 fixed_dot_8 zoom)
{
  fixed_dot_8 fixed_center_x, fixed_center_y ;

  fixed_center_x = (center_x * zoom) >> 8 ;
  fixed_center_y = (center_y * zoom) >> 8 ;

#define BG2_X_SCROLL TO_FIXED (240/2)
#define BG2_Y_SCROLL TO_FIXED (160/2)

  REG_BG2X =
    (BG2_X_SCROLL - fixed_center_y * sin_table[angle] -
     fixed_center_x * cos_table[angle]) ;
  REG_BG2Y =
    (BG2_Y_SCROLL - fixed_center_y * cos_table [angle] +
     fixed_center_x * sin_table [angle]) ;

  REG_BG2PA = (cos_table[angle] * zoom) >> 8 ;
  REG_BG2PB = (sin_table[angle] * zoom) >> 8 ;
  REG_BG2PC = (-sin_table[angle] * zoom) >> 8 ;
  REG_BG2PD = (cos_table[angle] * zoom) >> 8 ;
}




void ending_effect (u16 nb_palette_entries, u16 nb_tiles, const u16 *palette,
		    const u8 *tiles, const u8 *map)
{
  int i, j, k, l ;
  u16 old_screen_mode ;
  fixed_dot_8 scale ;
  u16 *tiles_memory = CHAR_BASE_BLOCK_2_ADDR ;
  u16 *video_memory = SCREEN_BASE_BLOCK_11_ADDR ;
  u16 *palette_memory = BACKGROUND_PALETTE_ADDR ;

  /* On sauvegarde le mode vido d'origine. */
  old_screen_mode = REG_DISPCNT ;
  set_screen_mode (0) ;

 /* Initialisation de la palette. */
  for (i = 0; i < nb_palette_entries; i++) palette_memory[i] = palette[i] ;

  /* Chargement des tiles du fond d'cran en mmoire. */
  for (i = 0; i < nb_tiles * (64 / 2); i++) {
    tiles_memory[i] =
      (u16) ((tiles[(i * 2) + 1]<< 8) | (tiles[(i * 2)])) ;
  }

  /* Effacement de toute la zone de map pour viter */
  /* avoir du garbage lors de la rotation.          */
  for (i = 0; i < (256 / 8) * (256 / 8) ; i++) video_memory[i] = (u16) 0 ;
  /* Chargement de la map de fond. */
  l = 0 ;
  k = 0 ;
  for (j = 0; j < 20; j++) {          /* Boucle verticale. */
    l = (j * (256 / 8)) / 2 ;
    for (i = 0; i < 30; i += 2) {        /* Boucle honrizontale. */
      video_memory[l] = (u16) ((map[k + 1] << 8) | map[k]) ;
      l++ ;
      k += 2 ;
    }
  }

  /* Layer de requesters divers de saisie. */
  set_background2 (BACKGROUND_PRIORITY_1 |
		   BACKGROUND_CHAR_BASE_BLOCK_2 |
		   BACKGROUND_COLOR_256 |
		   BACKGROUND_SCREEN_BASE_BLOCK_11 |
		   ROT_SCAL_BACKGROUND_256x256) ;

  scale = TO_FIXED(37) ;
  while (REG_VCOUNT < 160) ;
  rotate_background2 (i, 240/2, 160/2 , scale) ;
  set_screen_mode (MODE_1 | BG2_ENABLE) ;

  for (i = 0; i < 360; i += 5) {
    while (REG_VCOUNT < 160) ;
    rotate_background2 (i, 240/2, 160/2 , scale) ;
    if (i % 10 == 0) scale -= TO_FIXED(1) ;
  }

  /* On termine sur une image centre de taille normale. */
  while (REG_VCOUNT < 160) ;
  rotate_background2 (0, 240/2, 160/2 , TO_FIXED(1)) ;

  for (;;) {
    /* On attend une pression sur le bouton A, B */
    /* ou SELECT pour faire disparatre le tout. */
    KEY_POOL ;
    if ((KEY_HIT (BUTTON_A)) || (KEY_HIT (BUTTON_B)) ||
	(KEY_HIT (BUTTON_START))) break ;
  }

  set_screen_mode (0) ;

  /* On restaure les palettes. */
  init_palette () ;

  /* On restaure le mode vido d'origine. */
  set_screen_mode (old_screen_mode) ;
}





void manage_end_of_game () {

  short i, j ;

  play_sound (g_wav_end) ;
  /* Recherche d'un point d'entre dans la table des highscores. */
  for (i = 0; i < 5; i++) {
    if (current_game.score > current_hall_of_fame[i]) {
      /* Dcalage des scores infrieurs. */
      for (j = 4; j > i; j--)
	current_hall_of_fame[j] = current_hall_of_fame[j - 1] ;
      current_hall_of_fame[i] = current_game.score ;
      /* On sauve les nouveaux highscores. */
      save_fame () ;
      /* Un petit message de flicitation. */
      ending_effect (NB_WIN_PALETTE_ENTRIES, NB_WIN_TILES, win_palette,
		     win_tiles, win_map) ;
      return ;
    }
  }

  /* Pas de highscore battu. */
  ending_effect (NB_KO_PALETTE_ENTRIES, NB_KO_TILES, ko_palette, ko_tiles,
		 ko_map) ;
}
