/*
  *** Blocktrap GBA ***
 Franois PESSAUX September 2004
 This program is freeware.
 Use its sources as you whant provided that you leave this copyright unchanged
 and that you drop a small word telling that I'm the author of these sources.
 Thank's a lot.
   --Francois
*/

#include <stdlib.h>
#include <gba.h>
#include <timer.h>
#include <screen_mode.h>
#include <interrupt.h>
#include "game.h"
#include "blocks.h"
#include "background.h"
#include "init.h"
#include "print.h"
#include "control.h"
#include "ints.h"
#include "wav_laser.h"
#include "wav_turn.h"
#include "wav_color_elim.h"
#include "wav_eliminator.h"
#include "wav_perforator.h"
#include "wav_screen_cleaner.h"
#include "wav_color_mixer.h"
#include "wav_random_col.h"
#include "wav_random_dir.h"
#include "wav_timer_reset.h"
#include "wav_bonus_gained.h"
#include "wav_malus_gained.h"
#include "wav_eraser.h"



struct game current_game ;
unsigned int current_hall_of_fame[5] = { 0, 0, 0, 0, 0 };





u16 tile_from_block (block bl)
{
  u16 tile_info = 0 ;

  /* Dtermination de la forme. */
  switch (BLOCK_KIND (bl)) {
  case Block_None: tile_info = EMPTY_BLOCK_TILE_INDEX ; break ;
  case Block_Ko: tile_info = KO_BLOCK_TILE_INDEX ; break ;
  case Block_Regular: tile_info = FILLED_BLOCK_TILE_INDEX ; break ;
  case Block_Deep_charge: tile_info = DEEPCHARGE_UP_TILE_INDEX ; break ;
  case Block_Random_Direction: tile_info = RANDOM_DIRECTION_TILE_INDEX; break ;
  case Block_Inverted_Movement: tile_info = INVERTED_MVT_UP_TILE_INDEX; break ;
  case Block_Color_Cleaner: tile_info = COLOR_CLEANER_TILE_INDEX ; break ;
  case Block_Autofire: tile_info = AUTOFIRE_UP_TILE_INDEX ; break ;
  case Block_Painter: tile_info = PAINTER_UP_TILE_INDEX ; break ;
  case Block_Multidirection: tile_info = MULTIDIRECTION_TILE_INDEX ; break ;
  case Block_Line_Col_Cleaner:
    tile_info = LINE_COL_CLEANER_UP_TILE_INDEX ;
    break ;
  case Block_Screen_Cleaner: tile_info = SCREEN_CLEANER_TILE_INDEX ; break ;
  case Block_Random_Color: tile_info = RANDOM_COLOR_UP_TILE_INDEX ; break ;
  case Block_Color_Mixer: tile_info = COLOR_MIXER_TILE_INDEX ; break ;
  case Block_Furry_Moving: tile_info = FURY_MOVING_TILE_INDEX ; break ;
  case Block_Furry_Spinning: tile_info = FURY_SPINNING_TILE_INDEX ; break ;
  case Block_Blind: tile_info = BLIND_TILE_INDEX ; break ;
  case Block_Speed_Increase: tile_info = SPEED_INCREASE_TILE_INDEX ; break ;
  case Block_Speed_Decrease: tile_info = SPEED_DECREASE_TILE_INDEX ; break ;
  case Block_Antidote: tile_info = ANTIDOTE_TILE_INDEX ; break ;
  case Block_Eraser: tile_info = ERASER_TILE_INDEX ; break ;
  default:
    notify_long_message (FALSE, "tile_from_block\nASSERT 0") ;
    break ;
  }

  /* Dtermination de la couleur. */
  switch (BLOCK_COLOR (bl)) {
  case Blue: tile_info |= (BLUE_BLOCKS_PALETTE_SLOT << 12) ; break ;
  case Red: tile_info |= (RED_BLOCKS_PALETTE_SLOT << 12) ; break ;
  case Green: tile_info |= (GREEN_BLOCKS_PALETTE_SLOT << 12) ; break ;
  case Yellow: tile_info |= (YELLOW_BLOCKS_PALETTE_SLOT << 12) ; break ;
  case Indep: tile_info |= (COLOR_INDEP_BLOCKS_PALETTE_SLOT << 12) ; break ;
  default:
    notify_long_message (FALSE, "tile_from_block\nASSERT 1") ;
    break ;
  }

  return (tile_info) ;
}





void refresh_as_upper (unsigned short line_number)
{
  block *blocks ;
  short x_start, y, i ;
  u16 tile_info, tile_on_screen ;

  /* On remplit du haut vers le bas de l'cran en            */
  /* commenant par la fin du tableau reprsentant la ligne. */
  blocks = current_game.board.upper_lines[line_number].blocks ;

  x_start = UPPER_AREA_X + (line_number * 2) ;
  y = UPPER_AREA_Y ;
  for (i = LINE_SIZE - 1; i >= 0; i--) {
    tile_info = tile_from_block (blocks[i]) ;
    tile_on_screen =
      (y * (256 / 8)) /* Offset de ligne. */
      /* 256 : Largeur map en pixels. */
      /* 8 : Nbre pixels par tile. */
      + x_start ;  /* Offset de colonne. */
    playground_video_memory[tile_on_screen] = tile_info ;
    tile_on_screen++ ;
    playground_video_memory[tile_on_screen] = tile_info + 1 ;
    tile_on_screen =
      ((y + 1) * (256 / 8)) /* Offset de ligne. */
      /* 256 : Largeur map en pixels. */
      /* 8 : Nbre pixels par tile. */
      + x_start ;  /* Offset de colonne. */
    playground_video_memory[tile_on_screen] = tile_info + 2 ;
    tile_on_screen++ ;
    playground_video_memory[tile_on_screen] = tile_info + 3 ;
    y += 2 ;
  }
}



void refresh_as_left (unsigned short line_number)
{
  block *blocks ;
  short y_start, x, i ;
  u16 tile_info, tile_on_screen ;

  /* On remplit de la gauche vers la droite de l'cran en    */
  /* commenant par la fin du tableau reprsentant la ligne. */
  blocks = current_game.board.left_lines[line_number].blocks ;

  y_start = LEFT_AREA_Y + (line_number * 2) ;
  x = LEFT_AREA_X ;
  for (i = LINE_SIZE - 1; i >= 0; i--) {
    tile_info = tile_from_block (blocks[i]) ;
    tile_on_screen =
      (y_start * (256 / 8)) /* Offset de ligne. */
      /* 256 : Largeur map en pixels. */
      /* 8 : Nbre pixels par tile. */
      + x ;  /* Offset de colonne. */
    playground_video_memory[tile_on_screen] = tile_info ;
    tile_on_screen++ ;
    playground_video_memory[tile_on_screen] = tile_info + 1 ;
    tile_on_screen = ((y_start + 1) * (256 / 8)) + x ;
    playground_video_memory[tile_on_screen] = tile_info + 2 ;
    tile_on_screen++ ;
    playground_video_memory[tile_on_screen] = tile_info + 3 ;
    x += 2 ;
  }
}





void refresh_as_right (unsigned short line_number)
{
  block *blocks ;
  short y_start, x, i ;
  u16 tile_info, tile_on_screen ;

  /* On remplit de la gauche vers la droite de l'cran en      */
  /* commenant par led but du tableau reprsentant la ligne. */
  blocks = current_game.board.right_lines[line_number].blocks ;

  y_start = RIGHT_AREA_Y + (line_number * 2) ;
  x = RIGHT_AREA_X ;
  for (i = 0; i < LINE_SIZE; i++) {
    tile_info = tile_from_block (blocks[i]) ;
    tile_on_screen =
      (y_start * (256 / 8)) /* Offset de ligne. */
      /* 256 : Largeur map en pixels. */
      /* 8 : Nbre pixels par tile. */
      + x ;  /* Offset de colonne. */
    playground_video_memory[tile_on_screen] = tile_info ;
    tile_on_screen++ ;
    playground_video_memory[tile_on_screen] = tile_info + 1 ;
    tile_on_screen = ((y_start + 1) * (256 / 8)) + x ;
    playground_video_memory[tile_on_screen] = tile_info + 2 ;
    tile_on_screen++ ;
    playground_video_memory[tile_on_screen] = tile_info + 3 ;
    x += 2 ;
  }
}





void add_block_and_check_end_of_game
  (struct game *game,
   unsigned short rnd_line_number,
   enum direction direction_where_to_add,
   block new_block)
{
  struct line *area_where_to_insert = NULL ;
  block *line_where_to_insert = NULL ;
  short i ;

  switch (direction_where_to_add) {
  case North:
    /* On va rajouter dans la zone suprieure. */
    area_where_to_insert =  game->board.upper_lines ;
    break ;

  case West:
    /* On va rajouter dans la zone de gauche. */
    area_where_to_insert = game->board.left_lines ;
    break ;

  case East:
    /* On va rajouter dans la zone de droite. */
    area_where_to_insert = game->board.right_lines ;
    break ;
  default:
    notify_long_message
      (FALSE, "add_block_and_check_end_of_game\nASSERT 1") ;
    break ;
  }

  line_where_to_insert = area_where_to_insert[rnd_line_number].blocks ;

  /* On considre que la case la plus loigne */
  /* du centre est en fin de tableau.          */
  for (i = LINE_SIZE - 1; i >= 0; i--) {
    if (BLOCK_KIND (line_where_to_insert[i]) == Block_None) break ;
  }

  /* Test de saturation. */
  if (i < 0) {
    /* Ligne sature, fin de la partie. On marque la ligne incrimine. */
    for (i = 0; i < LINE_SIZE; i++)
      line_where_to_insert[i] =
	MAKE_BLOCK (Block_Ko, BLOCK_COLOR (line_where_to_insert[i])) ;
    game->game_running = FALSE ;
  }
  else {
    /* Sinon, on ajoute un nouveau bloc et jeu continue. */
    line_where_to_insert[i] = new_block ;
  }

  /* Et on rafrachit la grille modifie. */
  switch (direction_where_to_add) {
  case North:
    /* On va rafrachir la zone suprieure. */
    refresh_as_upper (rnd_line_number) ;
    break ;
  case West:
    /* On va rajouter dans la zone de gauche. */
    refresh_as_left (rnd_line_number) ;
    break ;
  case East:
    /* On va rajouter dans la zone de droite. */
    refresh_as_right (rnd_line_number) ;
    break ;
  default:
    notify_long_message
      (FALSE, "add_block_and_check_end_of_game\nASSERT 2") ;
    break ;
  }
}




short get_tempo_from_level (unsigned short level)
{
  if (level < 5) return (6) ;
  else
    if ((level >= 5) && (level < 10)) return (5) ;
    else
      if ((level >= 10) && (level < 15)) return (4) ;
      else return (3) ;
}





/* ******************************************************************** */
/* [Fonc] : change_level */
/* [Descr] : Fonction qui gre le changement de niveau. Elle incrmente */
/*    par effet de bord le niveau du jeu courant, met  jour la vitesse */
/*    d'apparition des blocs et le nouveau nombre de blocs restant     */
/*    dtruire pour accder au prochain niveau.                         */
/* ******************************************************************** */
void check_level_change (struct game *game)
{
  if (game->nb_blocks_remaining_to_destroy <= 0) {
    game->level++ ;
    game->temporisation = get_tempo_from_level (game->level) ;
    /* Chaque niveau demande 10  dtruire blocs de plus par */
    /* rapport au prcdent pour passer au niveau suprieur. */
    game->nb_blocks_remaining_to_destroy = 10 + game->level * 10 ;
  }
}





void flash_screen ()
{
  u16 old_blending_mode ;
  short i ;

  /* Petite anim. */
  old_blending_mode = REG_BLDMOD ;
  REG_COLEY = 0 ;
  set_blending_mode (FADE_MODE_2 | FADE_SOURCE_BG1) ;
  for (i = 0; i < 16; i++) {
    while (REG_VCOUNT < 160) ;
    REG_COLEY = (u16) i ;
    REG_BG1HOFS = ((s16) ((rand ()) % 8)) - 4 ;
    REG_BG1VOFS = ((s16) ((rand ()) % 8)) - 4 ;
  }
  for (i = 15; i >= 0; i--) {
    while (REG_VCOUNT < 160) ;
    REG_COLEY = (u16) i ;
    REG_BG1HOFS = ((s16) ((rand ()) % 8)) - 4 ;
    REG_BG1VOFS = ((s16) ((rand ()) % 8)) - 4 ;
  }

  REG_BG1HOFS = 0 ;
  REG_BG1VOFS = 0 ;
  set_blending_mode (old_blending_mode) ;
}





/* ************************************************************** */
/* [Fonc] : clear_color_in_line                                   */
/* [Descr] : Supprimer tous les blocs d'une certaine couleur dans */
/*             une ligne ou une colonne puis compacte les blocs   */
/* [Retour] : nombre de blocs dtruits dans la ligne ou colonne.  */
/* ************************************************************** */
u8 clear_color_in_line (block *line_where_to_search, enum color current_color)
{
  short i, j ;
  short nb_destroyed = 0 ;

  for (i = LINE_SIZE - 1; i >= 0;) {
    if (BLOCK_COLOR (line_where_to_search[i]) == current_color) {
      nb_destroyed++ ;
      /* On dcale, sauf si on est dj sur la premire case    */
      /* auquel cas on remplit avec la couleur None et on sort. */
      for (j = i; j > 0; j--)
	line_where_to_search[j] = line_where_to_search[j-1] ;
      line_where_to_search[0] = MAKE_BLOCK (Block_None, Indep) ;
    }
    else i-- ;
  }

  return (nb_destroyed) ;
}





void activate_screen_cleaner (struct game *game)
{
  short i, j ;
  short nb_destroyed = 0 ;

  for (i = 0; i < SQUARE_SIZE; i++) {
    for (j = 0; j < LINE_SIZE; j++) {
      if (BLOCK_KIND (game->board.upper_lines[i].blocks[j]) != Block_None)
	nb_destroyed++ ;
      if (BLOCK_KIND (game->board.left_lines[i].blocks[j]) != Block_None)
	nb_destroyed++ ;
      if (BLOCK_KIND (game->board.right_lines[i].blocks[j]) != Block_None)
	nb_destroyed++ ;
      game->board.upper_lines[i].blocks[j] = MAKE_BLOCK (Block_None, Indep) ;
      game->board.left_lines[i].blocks[j] = MAKE_BLOCK (Block_None, Indep) ;
      game->board.right_lines[i].blocks[j] = MAKE_BLOCK (Block_None, Indep) ;
    }
  }

  /* On met  jour le score. */
  game->score += (SIMPLE_SCORE_FACTOR * nb_destroyed) ;
  /* On met  jour le nombre de blocks restants. */
  game->nb_blocks_remaining_to_destroy -= nb_destroyed ;
  check_level_change (game) ;
  flash_screen () ;

  /* Rafrachissement initial du terrain. */
  for (i = 0; i < SQUARE_SIZE; i++) {
    refresh_as_upper (i) ;
    refresh_as_left (i) ;
    refresh_as_right (i) ;
  }
}





void activate_color_cleaner (struct game *game, enum color color)
{
  short nb_destroyed = 0 ;
  short i ;

  for (i = 0; i < LINE_SIZE; i++) {
    nb_destroyed +=
      clear_color_in_line (game->board.left_lines[i].blocks, color) ;
    nb_destroyed +=
      clear_color_in_line (game->board.upper_lines[i].blocks, color) ;
    nb_destroyed +=
      clear_color_in_line (game->board.right_lines[i].blocks, color) ;
  }

  /* On met  jour le score. */
  game->score += (SIMPLE_SCORE_FACTOR * nb_destroyed) ;
  /* On met  jour le nombre de blocks restants. */
  game->nb_blocks_remaining_to_destroy -= nb_destroyed ;
  check_level_change (game) ;
  flash_screen () ;

  /* Rafrachissement initial du terrain. */
  for (i = 0; i < SQUARE_SIZE; i++) {
    refresh_as_upper (i) ;
    refresh_as_left (i) ;
    refresh_as_right (i) ;
  }
}




void activate_color_mixer (struct game *game)
{
  enum color old_color ;
  enum block_kind old_kind ;
  short i, j ;

  for (i = 0; i < SQUARE_SIZE; i++) {
    for (j = 0; j < LINE_SIZE; j++) {
      /* Traitement de la partie haute du damier. */
      old_color = BLOCK_COLOR (game->board.upper_lines[i].blocks[j]) ;
      old_kind = BLOCK_KIND (game->board.upper_lines[i].blocks[j]) ;
      if ((old_kind != Block_None) && (old_color != Indep)) {
	old_color = (enum color) (((unsigned short) rand ()) % NB_COLORS) ;
	game->board.upper_lines[i].blocks[j] =
	  MAKE_BLOCK (old_kind, old_color) ;
      }
      /* Traitement de la partie gauche du damier. */
      old_color = BLOCK_COLOR (game->board.left_lines[i].blocks[j]) ;
      old_kind = BLOCK_KIND (game->board.left_lines[i].blocks[j]) ;
      if ((old_kind != Block_None) && (old_color != Indep)) {
	old_color = (enum color) (((unsigned short) rand ()) % NB_COLORS) ;
	game->board.left_lines[i].blocks[j] =
	  MAKE_BLOCK (old_kind, old_color) ;
      }
      /* Traitement de la partie droite du damier. */
      old_color = BLOCK_COLOR (game->board.right_lines[i].blocks[j]) ;
      old_kind = BLOCK_KIND (game->board.right_lines[i].blocks[j]) ;
      if((old_kind != Block_None) && (old_color != Indep)) {
	old_color = (enum color) (((unsigned short) rand ()) % NB_COLORS) ;
	game->board.right_lines[i].blocks[j] =
	  MAKE_BLOCK (old_kind, old_color) ;
      }
    }
  }

  flash_screen () ;

  /* Rafrachissement initial du terrain. */
  for (i = 0; i < SQUARE_SIZE; i++) {
    refresh_as_upper (i) ;
    refresh_as_left (i) ;
    refresh_as_right (i) ;
  }
}


/* boolen disant si le fait d'avoir shoot une block a fait changer par
   effet de bord la bullet du joueur. Doit tre remis  FALSE avant
   chaque tir. */
unsigned char bullet_enforced ;


/* Dclenche les ventuels effets ou stocke en rserve. */
void handle_shooted_block (struct game *game, block special_block)
{

  switch (BLOCK_KIND (special_block)) {
  case Block_Regular:
    /* Rien de particulier  faire. */
    break ;

  case Block_Deep_charge:
    play_sound (g_wav_bonus_gained) ;
    /*  mettre en rserve. */
    switch (BLOCK_COLOR (special_block)) {
    case Red:
      game->reserve[BONUS_Deep_Charge_Red_INDEX] =
	min (1 + game->reserve[BONUS_Deep_Charge_Red_INDEX], 99) ;
      break ;
    case Blue:
      game->reserve[BONUS_Deep_Charge_Blue_INDEX] =
	min (1 + game->reserve[BONUS_Deep_Charge_Blue_INDEX], 99) ;
      break ;
    case Green:
      game->reserve[BONUS_Deep_Charge_Green_INDEX] =
	min (1 + game->reserve[BONUS_Deep_Charge_Green_INDEX], 99) ;
      break ;
    case Yellow:
      game->reserve[BONUS_Deep_Charge_Yellow_INDEX] =
	min (1 + game->reserve[BONUS_Deep_Charge_Yellow_INDEX], 99) ;
      break ;
    default:
      notify_long_message (FALSE, "handle_shooted_special\nASSERT 0") ;
      break ;
    }
    break ;

  case Block_Random_Direction:
    /* Immdiatement actif. */
    play_sound (g_wav_malus_gained) ;
    bullet_enforced = 1 ;
    game->player.bullet = Bullet_Random_Direction ;
    break ;

  case Block_Inverted_Movement:
    /* Immdiatement actif et de dure limite. */
    play_sound (g_wav_malus_gained) ;
    game->player.status.status |= Status_Inverted_Movement ;
    game->player.status.inverted_movement.end =
      elapsed_ticks + ILLNESS_DURATION ;
    break ;

  case Block_Color_Cleaner:
    /* Immdiatement actif. */
    play_sound (g_wav_bonus_gained) ;
    activate_color_cleaner (game, BLOCK_COLOR (special_block)) ;
    break ;

  case Block_Autofire:
    /* Immdiatement actif et de dure limite. */
    play_sound (g_wav_malus_gained) ;
    game->player.status.status |= Status_Auto_Fire ;
    game->player.status.auto_fire.end = elapsed_ticks + ILLNESS_DURATION ;
    break ;

  case Block_Painter:
    /*  mettre en rserve. */
    play_sound (g_wav_bonus_gained) ;
    switch (BLOCK_COLOR (special_block)) {
    case Red:
      game->reserve[BONUS_Painter_Red_INDEX] =
	min (1 + game->reserve[BONUS_Painter_Red_INDEX], 99) ;
      break ;
    case Blue:
      game->reserve[BONUS_Painter_Blue_INDEX] =
	min (1 + game->reserve[BONUS_Painter_Blue_INDEX], 99) ;
      break ;
    case Green:
      game->reserve[BONUS_Painter_Green_INDEX] =
	min (1 + game->reserve[BONUS_Painter_Green_INDEX], 99) ;
      break ;
    case Yellow:
      game->reserve[BONUS_Painter_Yellow_INDEX] =
	min (1 + game->reserve[BONUS_Painter_Yellow_INDEX], 99) ;
      break ;
    default:
      notify_long_message (FALSE, "handle_shooted_special\nASSERT 1") ;
      break ;
    }
    break ;

  case Block_Multidirection:
    /* Immdiatement actif. */
    play_sound (g_wav_malus_gained) ;
    bullet_enforced = 1 ;
    game->player.bullet = Bullet_Multidirection ;
    break ;

  case Block_Line_Col_Cleaner:
    /*  mettre en rserve. */
    play_sound (g_wav_bonus_gained) ;
    game->reserve[BONUS_Line_Col_Cleaner_INDEX] =
      min (1 + game->reserve[BONUS_Line_Col_Cleaner_INDEX], 99) ;
    break ;

  case Block_Screen_Cleaner:
    /* Immdiatement actif. */
    play_sound (g_wav_bonus_gained) ;
    activate_screen_cleaner (game) ;
    break ;

  case Block_Random_Color:
    /* Immdiatement actif. */
    play_sound (g_wav_malus_gained) ;
    bullet_enforced = 1 ;
    game->player.bullet = Bullet_Random_Color ;
    break ;

  case Block_Color_Mixer:
    /* Immdiatement actif. */
    play_sound (g_wav_malus_gained) ;
    activate_color_mixer (game) ;
    break ;

  case Block_Furry_Moving:
    /* Immdiatement actif et de dure limite. */
    play_sound (g_wav_malus_gained) ;
    game->player.status.status |= Status_Furry_Moving ;
    game->player.status.furry_moving.end = elapsed_ticks + ILLNESS_DURATION ;
    break ;

  case Block_Furry_Spinning:
    /* Immdiatement actif et de dure limite. */
    play_sound (g_wav_malus_gained) ;
    game->player.status.status |= Status_Furry_Spinning ;
    game->player.status.furry_spinning.end = elapsed_ticks + ILLNESS_DURATION ;
    break ;

  case Block_Blind:
    /* Immdiatement actif et de dure limite. */
    play_sound (g_wav_malus_gained) ;
    game->player.status.status |= Status_Blind ;
    game->player.status.blind.end = elapsed_ticks + ILLNESS_DURATION ;
    break ;

  case Block_Speed_Increase:
    /* Immdiatement actif et de dure limite. */
    play_sound (g_wav_malus_gained) ;
    game->player.status.status |= Status_Speed_Increased ;
    game->player.status.speed_increased.end =
      elapsed_ticks + ILLNESS_DURATION ;
    game->temporisation = max (1, game->temporisation - 1) ;
    break ;

  case Block_Speed_Decrease:
    /* Immdiatement actif et de dure limite. */
    play_sound (g_wav_bonus_gained) ;
    game->player.status.status |= Status_Speed_Decreased ;
    game->player.status.speed_decreased.end =
      elapsed_ticks + ILLNESS_DURATION ;
    game->temporisation = game->temporisation + 1 ;
    break ;

  case Block_Antidote:
    /*  mettre en rserve. */
    play_sound (g_wav_bonus_gained) ;
    game->reserve[BONUS_Antidote_INDEX] =
      min (1 + game->reserve[BONUS_Antidote_INDEX], 99) ;
    break ;

  case Block_Eraser:
    /*  mettre en rserve. */
    play_sound (g_wav_bonus_gained) ;
    game->reserve[BONUS_Eraser_INDEX] =
      min (1 + game->reserve[BONUS_Eraser_INDEX], 99) ;
    break ;

  default:
    notify_long_message (FALSE, "handle_shooted_special\nASSERT 2") ;
    break ;
  }
}





/* Vrifie si un bloc est dtruit et si oui le dtruit. */
enum hit_result hit_block (struct game *game,
			   enum color expected_color,
			   block *line_where_to_search,
			   short index_in_line)
{
  enum color block_color ;

  if (BLOCK_KIND (line_where_to_search[index_in_line]) == Block_None)
    return (Cell_Empty) ;
  block_color = BLOCK_COLOR (line_where_to_search[index_in_line]) ;
  if ((block_color != expected_color) && (block_color != Indep))
    return (Cell_Not_Destroyed) ;

  /* Si c'est un bloc bonus, on l'active. */
  handle_shooted_block (game, line_where_to_search[index_in_line]) ;
  /* Comme c'est de la mme couleur ou un bonus, a disparait. */
  line_where_to_search[index_in_line] = MAKE_BLOCK (Block_None, Indep) ;

  return (Cell_Destroyed) ;
}





void refresh_according_to_player_direction (struct game *game)
{
  /* On raffrachit le terrain. */
  switch (game->player.direction) {
  case North:
    /* On va raffrachir la zone suprieure. */
    refresh_as_upper (game->player.x_pos) ;
    break ;
    
  case West:
    /* On va rajouter dans la zone de gauche. */
    refresh_as_left (game->player.y_pos) ;
    break ;

  case East:
    /* On va rajouter dans la zone de droite. */
    refresh_as_right (game->player.y_pos) ;
    break ;

  default:
    notify_long_message
      (FALSE, "refresh_according_to_player_direction\nASSERT 0") ;
    break ;
  }
}





/* Effectue un tir de bloc de comportement destructeur normal. */
enum color shoot_regular (struct game *game)
{
  enum color returned_color ;
  block *line_where_to_search = NULL ;
  short i ;
  short nb_destroyed ;
  unsigned short line_number = 0 ;

  /* Si on ne trouve rien, on retournera une couleur au hasard. */
  returned_color = (enum color) (((unsigned short) rand ()) % NB_COLORS) ;

  switch (game->player.direction) {
  case North:
    /* On va rechercher dans la zone suprieure. */
    line_where_to_search = game->board.upper_lines[game->player.x_pos].blocks ;
    line_number = game->player.x_pos ;
    break ;
  case West:
    /* On va rechercher dans la zone de gauche. */
    line_where_to_search = game->board.left_lines[game->player.y_pos].blocks ;
    line_number = game->player.y_pos ;
    break ;
  case East:
    /* On va rechercher dans la zone de droite. */
    line_where_to_search = game->board.right_lines[game->player.y_pos].blocks ;
    line_number = game->player.y_pos ;
    break ;
  default:
    notify_long_message (FALSE, "shoot_regular\nASSERT 0") ;
    break ;
  }

  /* On remonte jusqu' trouver un truc qui n'est pas de la mme */
  /* couleur et qui n'est pas un bloc indpendant de la couleur. */
  nb_destroyed = 0 ;
  for (i = 0; i < LINE_SIZE; i++) {
    switch (hit_block (game, game->player.color, line_where_to_search, i)) {
    case Cell_Empty:
      break ;

    case Cell_Destroyed:
      game->score += (1 << nb_destroyed) * MULTIPLE_SCORE_FACTOR ;
      nb_destroyed++ ;
      break ;

    case Cell_Not_Destroyed:
      returned_color = BLOCK_COLOR (line_where_to_search[i]) ;
      goto destructions_done ;
      break ;

    default:
      notify_long_message (FALSE, "shoot_regular\nASSERT 1") ;
      break ;
    }
  }

 destructions_done:
  /* Pnalisation si on a tir sur un block de couleur diffrente. */
  if (nb_destroyed == 0) {
    i = min (i, (LINE_SIZE - 1)) ;
    /* On attribut la couleur du bloc bloquant s'il y en avait. */
    if (BLOCK_KIND (line_where_to_search[i]) != Block_None)
      returned_color = BLOCK_COLOR (line_where_to_search[i]) ;
    /* Et maintenant, on rajoute le bloc du joueur dans la ligne. */
    add_block_and_check_end_of_game
      (game, line_number, game->player.direction,
       MAKE_BLOCK (Block_Regular, game->player.color)) ;
  }

  /* On met  jour le nombre de blocks restants. */
  game->nb_blocks_remaining_to_destroy -= nb_destroyed ;
  check_level_change (game) ;
  refresh_according_to_player_direction (game) ;
  return (returned_color) ;
}





block* find_line_from_player_position (struct game *game)
{
  switch (game->player.direction) {
  case North:
    /* On va rechercher dans la zone suprieure. */
    return (game->board.upper_lines[game->player.x_pos].blocks) ;
    break ;
  case West:
    /* On va rechercher dans la zone de gauche. */
    return (game->board.left_lines[game->player.y_pos].blocks) ;
    break ;
  case East:
    /* On va rechercher dans la zone de droite. */
    return (game->board.right_lines[game->player.y_pos].blocks) ;
    break ;
  default:
    notify_long_message (FALSE, "find_line_from_player_position\nASSERT 0") ;
    break ;
  }
  return (NULL) ;
}





/* ***************************************************************** */
/* [Fonc] : shoot_deep_charge                                        */
/* [Descr] : Effectue un tir de bloc dtruisant tous les blocs de la */
/*            couleur courante dans la ligne ou la colonne courante  */
/*            et compacte les blocs restants.                        */
/* ***************************************************************** */
void shoot_deep_charge (struct game *game)
{
  block *line_where_to_search = NULL ;
  short nb_destroyed = 0 ;

  line_where_to_search = find_line_from_player_position (game) ;
  nb_destroyed =
    clear_color_in_line (line_where_to_search, game->player.color) ;

  /* On met  jour le score. */
  game->score += (1 << nb_destroyed) * MULTIPLE_SCORE_FACTOR ;
  /* On met  jour le nombre de blocks restants. */
  game->nb_blocks_remaining_to_destroy -= nb_destroyed ;
  check_level_change (game) ;
  refresh_according_to_player_direction (game) ;
}





void shoot_line_col_cleaner (struct game *game)
{
  block *line_where_to_search = NULL ;
  short i ;
  short nb_destroyed ;

  line_where_to_search = find_line_from_player_position (game) ;
  nb_destroyed = 0 ;
  for (i = 0; i < LINE_SIZE; i++) {
    if (BLOCK_KIND (line_where_to_search[i]) != Block_None) {
      nb_destroyed++ ;
      line_where_to_search[i] = MAKE_BLOCK (Block_None, Indep) ;
    }
  }

  /* On met  jour le score. */
  game->score += (1 << nb_destroyed) * MULTIPLE_SCORE_FACTOR ;
  /* On met  jour le nombre de blocks restants. */
  game->nb_blocks_remaining_to_destroy -= nb_destroyed ;
  check_level_change (game) ;
  refresh_according_to_player_direction (game) ;
}





void shoot_painter (struct game *game)
{
  block *line_where_to_search = NULL ;
  enum block_kind kind ;
  enum color color ;
  short i ;
  
  line_where_to_search = find_line_from_player_position (game) ;
  for (i = 0; i < LINE_SIZE; i++) {
    kind = BLOCK_KIND (line_where_to_search[i]) ;
    if (kind != Block_None) {
      color = BLOCK_COLOR (line_where_to_search[i]) ;
      if (color != Indep)
	line_where_to_search[i] = MAKE_BLOCK (kind, game->player.color) ;
    }
  }

  refresh_according_to_player_direction (game) ;
}





void shoot_eraser (struct game *game)
{
  block *line_where_to_search = NULL ;
  short nb_destroyed = 0 ;
  short i ;

  line_where_to_search = find_line_from_player_position (game) ;
  /* On remonte jusqu' trouver un bloc qui n'est pas vide. */
  for (i = 0; i < LINE_SIZE; i++) {
    if (BLOCK_KIND (line_where_to_search[i]) != Block_None) {
      line_where_to_search[i] = MAKE_BLOCK (Block_None, Indep) ;
      nb_destroyed++ ;
      break ;
    }
  }

  /* On met  jour le nombre de blocks restants. */
  game->nb_blocks_remaining_to_destroy -= nb_destroyed ;
  check_level_change (game) ;
  refresh_according_to_player_direction (game) ;
}





enum color main_bullet_shooter (struct game *game)
{
  enum color returned_color ;

  returned_color = game->player.color ;

  switch (game->player.bullet) {
  case Bullet_Regular:
    play_sound (g_wav_laser) ;
    returned_color = shoot_regular (game) ;
    break ;

  case Bullet_Deep_Charge:
    play_sound (g_wav_eliminator) ;
    shoot_deep_charge (game) ;
    /* On retourne une nouvelle couleur au hasard. */
    returned_color = (enum color) (((unsigned short) rand ()) % NB_COLORS) ;
    break ;

  case Bullet_Random_Direction:
    play_sound (g_wav_random_dir) ;
    game->player.direction =
      (enum direction) (((unsigned short) rand ()) % 3) ;
    returned_color = shoot_regular (game) ;
    break ;

  case Bullet_Painter:
    shoot_painter (game) ;
    /* On retourne une nouvelle couleur au hasard. */
    returned_color = (enum color) (((unsigned short) rand ()) % NB_COLORS) ;
    break ;

  case Bullet_Multidirection:
    {
      enum direction direction_backup ;

      direction_backup = game->player.direction ;
      game->player.direction = West ;
      shoot_regular (game) ;
      game->player.direction = North ;
      shoot_regular (game) ;
      game->player.direction = East ;
      shoot_regular (game) ;
      game->player.direction = direction_backup ;
      /* On retourne une nouvelle couleur au hasard. */
      returned_color = (enum color) (((unsigned short) rand ()) % NB_COLORS) ;
    }
    break ;

  case Bullet_Line_Col_Cleaner:
    play_sound (g_wav_perforator) ;
    shoot_line_col_cleaner (game) ;
    /* On retourne une nouvelle couleur au hasard. */
    returned_color = (enum color) (((unsigned short) rand ()) % NB_COLORS) ;
    break ;

  case Bullet_Random_Color:
    play_sound (g_wav_random_col) ;
    game->player.color =
      (enum color) (((unsigned short) rand ()) % NB_COLORS) ;
    returned_color = shoot_regular (game) ;
    break ;

  case Bullet_Eraser:
    play_sound (g_wav_eraser) ;
    shoot_eraser (game) ;
    /* On retourne une nouvelle couleur au hasard. */
    returned_color = (enum color) (((unsigned short) rand ()) % NB_COLORS) ;
    break ;

  default:
    notify_long_message (FALSE, "main_bullet_shooter\nASSERT 0") ;
    break ;
  }

  return (returned_color) ;
}





void draw_player_position (struct game *game)
{
  u16 tile_on_screen ;
  u16 tile_info = 0 ;

  tile_on_screen =
    (((game->player.y_pos * 2) + CENTRAL_AREA_Y) * (256 / 8))
    + ((game->player.x_pos * 2) + CENTRAL_AREA_X) ;

  /* Dtermination de la couleur. */
  switch (game->player.color) {
  case Blue: tile_info |= (BLUE_BLOCKS_PALETTE_SLOT << 12) ; break ;
  case Red: tile_info |= (RED_BLOCKS_PALETTE_SLOT << 12) ; break ;
  case Green: tile_info |= (GREEN_BLOCKS_PALETTE_SLOT << 12) ; break ;
  case Yellow: tile_info |= (YELLOW_BLOCKS_PALETTE_SLOT << 12) ; break ;
  case Indep: tile_info |= (COLOR_INDEP_BLOCKS_PALETTE_SLOT << 12) ; break ;
  default:
    notify_long_message (FALSE, "draw_player_position\nASSERT 0") ;
    break ;
  }

  /* Dtermination de la forme. */
  switch (game->player.bullet) {
  case Bullet_Regular:
    switch (game->player.direction) {
    case North: tile_info |= NORMAL_UP_TILE_INDEX ; break ;
    case West: tile_info |= NORMAL_LEFT_TILE_INDEX ; break ;
    case East: tile_info |= NORMAL_RIGHT_TILE_INDEX ; break ;
    default:
      notify_long_message (FALSE, "draw_player_position\nASSERT 1") ;
      break ;
    }
    break ;

  case Bullet_Deep_Charge:
    switch (game->player.direction) {
    case North: tile_info |= DEEPCHARGE_UP_TILE_INDEX ; break ;
    case West: tile_info |= DEEPCHARGE_LEFT_TILE_INDEX ; break ;
    case East: tile_info |= DEEPCHARGE_RIGHT_TILE_INDEX ; break ;
    default:
      notify_long_message (FALSE, "draw_player_position\nASSERT 2") ;
      break ;
    }
    break ;

  case Bullet_Random_Direction:
    tile_info |= RANDOM_DIRECTION_TILE_INDEX ;
    break ;

  case Bullet_Painter:
    switch (game->player.direction) {
    case North: tile_info |= PAINTER_UP_TILE_INDEX ; break ;
    case West: tile_info |= PAINTER_LEFT_TILE_INDEX ; break ;
    case East: tile_info |= PAINTER_RIGHT_TILE_INDEX ; break ;
    default:
      notify_long_message (FALSE, "draw_player_position\nASSERT 3") ;
      break ;
    }
    break ;

  case Bullet_Multidirection:
    tile_info |= MULTIDIRECTION_TILE_INDEX ;
    break ;

  case Bullet_Line_Col_Cleaner:
    switch (game->player.direction) {
    case North:
      tile_info =
	PLAYER_LINE_COL_CLEANER_UP_TILE_INDEX |
	(COLOR_INDEP_BLOCKS_PALETTE_SLOT << 12) ;
      break ;
    case West:
      tile_info =
	PLAYER_LINE_COL_CLEANER_LEFT_TILE_INDEX |
	(COLOR_INDEP_BLOCKS_PALETTE_SLOT << 12) ;
      break ;
    case East:
      tile_info =
	PLAYER_LINE_COL_CLEANER_RIGHT_TILE_INDEX |
	(COLOR_INDEP_BLOCKS_PALETTE_SLOT << 12) ;
      break ;
    default:
      notify_long_message (FALSE, "draw_player_position\nASSERT 4") ;
      break ;
    }
    break ;

  case Bullet_Random_Color:
    switch (game->player.direction) {
    case North:
      tile_info =
	RANDOM_COLOR_UP_TILE_INDEX | (COLOR_INDEP_BLOCKS_PALETTE_SLOT << 12) ;
      break ;
    case West:
      tile_info =
	RANDOM_COLOR_LEFT_TILE_INDEX |
	(COLOR_INDEP_BLOCKS_PALETTE_SLOT << 12) ;
      break ;
    case East:
      tile_info =
	RANDOM_COLOR_RIGHT_TILE_INDEX |
	(COLOR_INDEP_BLOCKS_PALETTE_SLOT << 12) ;
      break ;
    default:
      notify_long_message (FALSE, "draw_player_position\nASSERT 5") ;
      break ;
    }
    break ;

  case Bullet_Eraser:
    switch (game->player.direction) {
    case North:
      tile_info |= ERASER_UP_TILE_INDEX  ;
      break ;
    case West:
      tile_info |= ERASER_LEFT_TILE_INDEX  ;
      break ;
    case East:
      tile_info |= ERASER_RIGHT_TILE_INDEX  ;
      break ;
    default:
      notify_long_message (FALSE, "draw_player_position\nASSERT 6") ;
      break ;
    }
    break ;

  default:
    notify_long_message (FALSE, "draw_player_position\nASSERT 7") ;
    break ;
  }

  playground_video_memory[tile_on_screen] = tile_info ;
  tile_on_screen++ ;
  playground_video_memory[tile_on_screen] = tile_info + 1 ;
  tile_on_screen =
    ((((game->player.y_pos  * 2) + 1) + CENTRAL_AREA_Y) * (256 / 8)) +
    ((game->player.x_pos * 2) + CENTRAL_AREA_X) ;
  playground_video_memory[tile_on_screen] = tile_info + 2 ;
  tile_on_screen++ ;
  playground_video_memory[tile_on_screen] = tile_info + 3 ;
}





void analyse_player_input (struct game *game, enum action action)
{
  short old_x, old_y ;
  u16 tile_on_screen, tile_info ;

  old_x = game->player.x_pos ;
  old_y = game->player.y_pos ;

  switch (action) {
  case Go_up:
    if (game->player.y_pos > 0) game->player.y_pos-- ;
    break ;

  case Go_down:
    if (game->player.y_pos < SQUARE_SIZE - 1) game->player.y_pos++ ;
    break ;

  case Go_left:
    if (game->player.x_pos > 0) game->player.x_pos-- ;
    break ;

  case Go_right:
    if (game->player.x_pos < SQUARE_SIZE - 1) game->player.x_pos++ ;
    break ;

  case Turn_clockwise:
    play_sound (g_wav_turn) ;
    game->player.direction = (game->player.direction + 1) % 3 ;
    break ;

  case Turn_reverse:
    play_sound (g_wav_turn) ;
    if (game->player.direction == 0) game->player.direction = 2 ;
    else game->player.direction = (game->player.direction - 1) % 3 ;
    break ;

  case Fire:
    bullet_enforced = 0 ;
    game->player.color = main_bullet_shooter (game) ;
    /* On ne rtablit le type de bullet que s'il n'a pas t forc */
    /* par l'effet du tir (mise  jour par effet de bord).         */
    if (!bullet_enforced) game->player.bullet = Bullet_Regular ;
    break ;

  case Invoke_bonus:
    reserve_selection (game) ;
    break ;

  default:
    notify_long_message (FALSE, "analyse_player_input\nASSERT 0") ;
    break ;
  }

  /* On efface l'ancienne position dans la zone centrale. */
  tile_info = EMPTY_BLOCK_TILE_INDEX |
    (COLOR_INDEP_BLOCKS_PALETTE_SLOT << 12) ;
  tile_on_screen =
    (((old_y * 2)+ CENTRAL_AREA_Y) * (256 / 8)) /* Offset de ligne. */
					   /* 256 : Largeur map en pixels. */
					   /* 8 : Nbre pixels par tile. */
    + ((old_x * 2) + CENTRAL_AREA_X) ;           /* Offset de colonne. */
  playground_video_memory[tile_on_screen] = tile_info ;
  tile_on_screen++ ;
  playground_video_memory[tile_on_screen] = tile_info + 1 ;
  tile_on_screen =
    ((((old_y * 2) + 1) + CENTRAL_AREA_Y) * (256 / 8)) +
    ((old_x * 2) + CENTRAL_AREA_X) ;
  playground_video_memory[tile_on_screen] = tile_info + 2 ;
  tile_on_screen++ ;
  playground_video_memory[tile_on_screen] = tile_info + 3 ;

  /* On dessine la nouvelle position dans la zone centrale. */
  draw_player_position (game) ;
  print_reserve (game->reserve) ;
  print_score (game->score) ;
}





/* Utilise lorsque l'on recharge un jeu. */
void initial_full_refresh (struct game *game)
{
  short i, j ;
  u16 tile_on_screen, tile_info ;

  /* Effacement de la zone centrale. */
  for (i = 0; i < SQUARE_SIZE; i++) {
    for (j = 0; j < SQUARE_SIZE; j++) {
      tile_info =
	EMPTY_BLOCK_TILE_INDEX | (COLOR_INDEP_BLOCKS_PALETTE_SLOT << 12) ;
      tile_on_screen =
	(((j * 2) + CENTRAL_AREA_Y) * (256 / 8)) + ((i * 2) + CENTRAL_AREA_X) ;
      playground_video_memory[tile_on_screen] = tile_info ;
      tile_on_screen++ ;
      playground_video_memory[tile_on_screen] = tile_info + 1 ;
      tile_on_screen =
	((((j * 2) +1) + CENTRAL_AREA_Y) * (256 / 8)) +
	((i * 2) + CENTRAL_AREA_X) ;
      playground_video_memory[tile_on_screen] = tile_info + 2 ;
      tile_on_screen++ ;
      playground_video_memory[tile_on_screen] = tile_info + 3 ;
    }
  }

  /* Rafrachissement initial du terrain. */
  for (i = 0; i < SQUARE_SIZE; i++) {
    refresh_as_upper (i) ;
    refresh_as_left (i) ;
    refresh_as_right (i) ;
  }

  /* Affichage du score. */
  print_score (game->score) ;
  /* Affichage de la position du joueur dans la zone centrale. */
  /* On dessine la nouvelle position dans la zone centrale. */
  draw_player_position (game) ;
  /* Affichage de la zone des bonus. */
  print_reserve (game->reserve) ;
}





struct game* new_game (short bonus_proba)
{
  short i, j ;

  current_game.player.x_pos = 0 ;
  current_game.player.y_pos = 0 ;
  current_game.player.direction = North ;
  current_game.player.bullet = Bullet_Regular ;
  current_game.player.color =
    (enum color) (((unsigned short) rand ()) % NB_COLORS) ;
  current_game.player.status.status = Status_Normal ;

  current_game.player.status.inverted_movement.end = 0L ;
  current_game.player.status.inverted_movement.previous_action = 0L ;
  current_game.player.status.blind.end = 0L ;
  current_game.player.status.blind.previous_action = 0L ;
  current_game.player.status.furry_moving.end = 0L ;
  current_game.player.status.furry_moving.previous_action = 0L ;
  current_game.player.status.furry_spinning.end = 0L ;
  current_game.player.status.furry_spinning.previous_action = 0L ;
  current_game.player.status.auto_fire.end = 0L ;
  current_game.player.status.auto_fire.previous_action = 0L ;
  current_game.player.status.speed_increased.end = 0L ;
  current_game.player.status.speed_increased.previous_action = 0L ;
  current_game.player.status.speed_decreased.end = 0L ;
  current_game.player.status.speed_decreased.previous_action = 0L ;

  current_game.nb_blocks_remaining_to_destroy = 10 ;
  current_game.percent_chances_of_bonus = bonus_proba ;
  current_game.level = 0 ;
  current_game.score = 0 ;
  current_game.game_running = TRUE ;
  current_game.temporisation = get_tempo_from_level (current_game.level) ;

  for (i = 0; i < BONUS_RESERVE_SIZE; i++) current_game.reserve[i] = 0 ;

  for (i = 0; i < SQUARE_SIZE; i++) {
    for (j = 0; j < LINE_SIZE; j++) {
      current_game.board.upper_lines[i].blocks[j] =
	MAKE_BLOCK (Block_None, Indep) ;
      current_game.board.left_lines[i].blocks[j] =
	MAKE_BLOCK (Block_None, Indep) ;
      current_game.board.right_lines[i].blocks[j] =
	MAKE_BLOCK (Block_None, Indep) ;
    }
  }

  /* Ajout d'un block de dpart. */
  add_block_and_check_end_of_game
    (&current_game,
     ((unsigned short) rand ()) % SQUARE_SIZE,
     (enum direction) (((unsigned short) rand ()) % 3),
     (enum color) (((unsigned short) rand ()) % NB_COLORS)) ;

  initial_full_refresh (&current_game) ;
  return (&current_game) ;
}
