/*
  *** Blocktrap GBA ***
 Franois PESSAUX September 2004
 This program is freeware.
 Use its sources as you whant provided that you leave this copyright unchanged
 and that you drop a small word telling that I'm the author of these sources.
 Thank's a lot.
   --Francois
*/

#ifndef __GAME_H
#define __GAME_H

typedef unsigned char BOOL ;
#define FALSE 0
#define TRUE 1

#define min(a,b) ((a) < (b) ? (a) : (b))
#define max(a,b) ((a) > (b) ? (a) : (b))

enum direction {
  North,
  East,
  West
} ;

enum color {
  Blue = 0,
  Red = 1,
  Green = 2,
  Yellow = 3,
  Indep = 4
} ;

#define NB_BLOCK_KINDS 21
#define NB_NON_SPECIAL_KINDS 3
enum block_kind {
  Block_None = 0,
  Block_Ko = 1,
  Block_Regular = 2,
  Block_Deep_charge = 3,
  Block_Random_Direction = 4,
  Block_Inverted_Movement = 5,
  Block_Color_Cleaner = 6,
  Block_Autofire = 7,
  Block_Painter = 8,
  Block_Multidirection = 9,
  Block_Line_Col_Cleaner = 10,
  Block_Screen_Cleaner = 11,
  Block_Random_Color = 12,
  Block_Color_Mixer = 13,
  Block_Furry_Moving = 14,
  Block_Furry_Spinning = 15,
  Block_Blind = 16,
  Block_Speed_Increase = 17,
  Block_Speed_Decrease = 18,
  Block_Antidote = 19,
  Block_Eraser = 20,
} ;

enum action {
  Go_up = 0,
  Go_down = 1,
  Go_left = 2,
  Go_right = 3,
  Turn_clockwise = 4,
  Turn_reverse = 5,
  Fire = 6,
  Invoke_bonus = 7
} ;



#define Status_Normal 0
#define Status_Inverted_Movement 1
#define Status_Blind (1 << 1)
#define Status_Furry_Moving (1 << 2)
#define Status_Furry_Spinning (1 << 3)
#define Status_Auto_Fire (1 << 4)
#define Status_Speed_Increased (1 << 5)
#define Status_Speed_Decreased (1 << 6)


struct effect {
  unsigned long end ;     /* Stamp de fin. */
  unsigned long previous_action ;  /* Dernier stamp auquel l'effet a jou. */
} ;


struct status {
  u16 status ;
  struct effect inverted_movement ;
  struct effect blind ;
  struct effect furry_moving ;
  struct effect furry_spinning ;
  struct effect auto_fire ;
  struct effect speed_increased ;
  struct effect speed_decreased ;
} ;


enum bullet {
  Bullet_Regular = 0,
  Bullet_Deep_Charge = 1,
  Bullet_Random_Direction = 2,
  Bullet_Painter = 3,
  Bullet_Multidirection = 4,
  Bullet_Line_Col_Cleaner = 5,
  Bullet_Random_Color = 6,
  Bullet_Eraser = 7
} ;


enum hit_result {
  Cell_Empty,
  Cell_Destroyed,
  Cell_Not_Destroyed
} ;

#define SQUARE_SIZE 4
#define LINE_SIZE 4
#define NB_COLORS 4
#define PERCENT_CHANCES_OF_BONUS 25

typedef u16 block ;

#define BLOCK_KIND(b) ((enum block_kind) (((b) & 0xFF00) >> 8))
#define BLOCK_COLOR(b) ((enum color) ((b) & 0x00FF))
#define MAKE_BLOCK(kind,color) ((block) (((kind) << 8) | (color)))

/* La case la plus loigne du centre est en fin de tableau. */
struct line {
  block blocks[LINE_SIZE] ;
} ;

struct board {
  struct line upper_lines[SQUARE_SIZE] ;
  struct line left_lines[SQUARE_SIZE] ;
  struct line right_lines[SQUARE_SIZE] ;
};

#define ILLNESS_DURATION 20
struct player {
  short x_pos ;
  short y_pos ;
  enum direction direction ;
  enum bullet bullet ;
  enum color color ;
  struct status status ;
} ;

#define BONUS_RESERVE_SIZE 11

#define BONUS_Deep_Charge_Red_INDEX 0
#define BONUS_Deep_Charge_Blue_INDEX 1
#define BONUS_Deep_Charge_Green_INDEX 2
#define BONUS_Deep_Charge_Yellow_INDEX 3

#define BONUS_Painter_Red_INDEX 4
#define BONUS_Painter_Blue_INDEX 5
#define BONUS_Painter_Green_INDEX 6
#define BONUS_Painter_Yellow_INDEX 7

#define BONUS_Line_Col_Cleaner_INDEX 8
#define BONUS_Antidote_INDEX 9
#define BONUS_Eraser_INDEX 10

#define SIMPLE_SCORE_FACTOR 5
#define MULTIPLE_SCORE_FACTOR 10

struct game {
  int nb_blocks_remaining_to_destroy ;
  short percent_chances_of_bonus ;
  unsigned short level ;
  unsigned int score ;
  BOOL game_running ;
  short temporisation ;
  unsigned char reserve[BONUS_RESERVE_SIZE] ;
  struct player player ;
  struct board board ;
} ;

extern struct game current_game ;
extern unsigned int current_hall_of_fame[5] ;

struct game* new_game (short) ;
void initial_full_refresh (struct game*) ;
void add_block_and_check_end_of_game
  (struct game*, unsigned short, enum direction, block) ;
short get_tempo_from_level (unsigned short level) ;
void analyse_player_input (struct game*, enum action) ;
void draw_timer_frame (u16) ;

struct saved_data {
  unsigned long int magic_tag ;
  int rand_new_seed ;
  unsigned int saved_hall_of_fame[5] ;
  struct game saved_game ;
};


#define TAGTOLONG(a,b,c,d) ((unsigned long)(((a)<<24)|((b)<<16)|((c)<<8)|(d)))
#define MAGICNUMBER TAGTOLONG('B','L','T','P')

#endif
