#include <math.h>
#include <gba.h>
#include <screen_mode.h>
#include "game.h"
#include "background.h"
#include "digits.h"
#include "blocks.h"
#include "fonts.h"
#include "requesters_gfx.h"
#include "print.h"
#include "cursor_gfx.h"
#include "init.h"


u16 *background_video_memory = SCREEN_BASE_BLOCK_8_ADDR ;
u16 *playground_video_memory = SCREEN_BASE_BLOCK_9_ADDR ;
u16 *requesters_video_memory = SCREEN_BASE_BLOCK_10_ADDR ;
struct sprite *sprite_video_memory = SPRITES_INFO_MEMORY_ADR ;
struct sprite cursor_sprite = { 0, 0, 0, 0 } ;



void setup_cursor_graphics (void)
{
  u16 i ;
  u16 *sprites_tiles_memory = SPRITES_CHAR_BASE_BLOCK_ADR ;
  u16 *sprite_palette = SPRITES_PALETTE_ADR ;

  /* Chargement des tiles des sprites. */
  for (i=0; i< NB_CURSOR_TILES * (32 / 2) ; i++) 
    sprites_tiles_memory[i] =
      (u16) ((cursor_tiles[(i * 2) + 1]<< 8) | (cursor_tiles[(i * 2)])) ;

  /* Chargement de la palette des sprites. */
  for (i = 0; i < 16 ; i++)
    sprite_palette[i] = cursor_palette[i] ;

  /* Cache tous les sprites en dehors de l'ecran pour que */
  /* ceux que l'on n'utilisera pas ne se voient pas.      */
  for (i=0; i<128; i++) {
    sprite_video_memory[i].attribute0 = 160 ; /* y vers > 159. */
    sprite_video_memory[i].attribute1 = 240 ; /* x vers  > 239. */
  }

  /* Initialisation du sprite du curseur. */
  cursor_sprite.attribute0 = SPRITE_16_COLOR | SPRITE_SQUARE ;
  cursor_sprite.attribute1 = SPRITE_SIZE_16 ;
  cursor_sprite.attribute2 = CURSOR_SPRITE_TILE_INDEX ;
}





void init_palette ()
{
  u16 *palette_memory = BACKGROUND_PALETTE_ADDR ;
  u16 i, palette_index ;

  palette_index = 16 * 0 ;
  /* Chargement de la palette de fond en slot 0. */
  for (i = 0; i < 16; i++) {
    palette_memory[palette_index] = background_palette[i] ;
    palette_index++ ;
  }

  palette_index = 16 * 1 ;
  /* Chargement de la palette des requester en slot 1 .*/
  for (i = 0; i < 16; i++) {
    palette_memory[palette_index] = requester_palette[i] ;
    palette_index++ ;
  }

  palette_index = 16 * 2 ;
  /* Chargement de la palette pour les blocs bleus en slot 2. */
  for (i = 0; i < NB_COLOR_DEP_PALETTE_ENTRIES; i++) {
    palette_memory[palette_index] = blue_block_palette[i] ;
    palette_index++ ;
  }

  palette_index = 16 * 3 ;
  /* Chargement de la palette pour les blocs rouges en slot 3. */
  for (i = 0; i < NB_COLOR_DEP_PALETTE_ENTRIES; i++) {
    palette_memory[palette_index] = red_block_palette[i] ;
    palette_index++ ;
  }

  palette_index = 16 * 4 ;
  /* Chargement de la palette pour les blocs verts en slot 4. */
  for (i = 0; i < NB_COLOR_DEP_PALETTE_ENTRIES; i++) {
    palette_memory[palette_index] = green_block_palette[i] ;
    palette_index++ ;
  }

  palette_index = 16 * 5 ;
  /* Chargement de la palette pour les blocs jaunes en slot 5. */
  for (i = 0; i < NB_COLOR_DEP_PALETTE_ENTRIES; i++) {
    palette_memory[palette_index] = yellow_block_palette[i] ;
    palette_index++ ;
  }

  palette_index = 16 * 6 ;
  /* Chargement de la palette pour les blocs indpendants */
  /* de la couleur du joueur en slot 6.                   */
  for (i = 0; i < NB_COLOR_INDEP_PALETTE_ENTRIES; i++) {
    palette_memory[palette_index] = color_indep_blocks_palette[i] ;
    palette_index++ ;
  }

  /* Pas d'init ici la palette des requesters de saisie divers (slot 7). */
  
  palette_index = 16 * 8 ;
  /* Chargement de la palette pour les digits de nombres de */
  /* bonus indpendants de la couleur  */
  /* de la couleur du joueur en slot 7.                   */
  for (i = 0; i < NB_COLOR_INDEP_DIGITS_PALETTE_ENTRIES; i++) {
    palette_memory[palette_index] = color_indep_digits_blocks_palette[i] ;
    palette_index++ ;
  }
}




void init_playground (void)
{
  u16 *tiles_memory = CHAR_BASE_BLOCK_0_ADDR ;
  u16 tile_index, map_index ;
  u16 i, j ;

  /* Layer de fond. */
  set_background0 (BACKGROUND_PRIORITY_3 |
		   BACKGROUND_CHAR_BASE_BLOCK_0 |
		   BACKGROUND_COLOR_16 |
		   BACKGROUND_SCREEN_BASE_BLOCK_8 |
		   TEXT_BACKGROUND_256x256) ;
  /* Layer de jeu. */
  set_background1 (BACKGROUND_PRIORITY_2 |
		   BACKGROUND_CHAR_BASE_BLOCK_0 |
		   BACKGROUND_COLOR_16 |
		   BACKGROUND_SCREEN_BASE_BLOCK_9 |
		   TEXT_BACKGROUND_256x256) ;

  /* Layer de requesters divers de saisie. */
  set_background2 (BACKGROUND_PRIORITY_1 |
		   BACKGROUND_CHAR_BASE_BLOCK_2 |
		   BACKGROUND_COLOR_16 |
		   BACKGROUND_SCREEN_BASE_BLOCK_11 |
		   TEXT_BACKGROUND_256x256) ;

  /* Layer de requester d'affichage de message. */
  set_background3 (BACKGROUND_PRIORITY_0 |
		   BACKGROUND_CHAR_BASE_BLOCK_0 |
		   BACKGROUND_COLOR_16 |
		   BACKGROUND_SCREEN_BASE_BLOCK_10 |
		   TEXT_BACKGROUND_256x256) ;

  tile_index = 0 ;
  /* Chargement des tiles de fond en mmoire. */
  for (i = 0; i < NB_BACKGROUND_TILES * (32 / 2); i++) {
    tiles_memory[tile_index] =
      (u16) ((background_tiles[(i * 2) + 1] << 8) |
	     (background_tiles[(i * 2)])) ;
    tile_index++ ;
  }

  /* Chargement des tiles de blocs dpendants */
  /* de la couleur du joueur en mmoire.      */
  for (i = 0; i < NB_COLORED_BLOCKS_TILES * (32 / 2); i++) {
    tiles_memory[tile_index] =
      (u16) ((colored_blocks_tiles[(i * 2) + 1] << 8) |
	     (colored_blocks_tiles[(i * 2)])) ;
    tile_index++ ;
  }

  /* Chargement des tiles de blocs indpendants */
  /* de la couleur du joueur en mmoire.        */
  for (i = 0; i < NB_COLOR_INDEP_BLOCKS_TILES * (32 / 2); i++) {
    tiles_memory[tile_index] =
      (u16) ((color_indep_blocks_tiles[(i * 2) + 1] << 8) |
	     (color_indep_blocks_tiles[(i * 2)])) ;
    tile_index++ ;
  }

  /* Chargement des tiles de chiffres de score en mmoire. */
  for (i = 0; i < NB_DIGITS_TILES * (32 / 2); i++) {
    tiles_memory[tile_index] =
      (u16) ((digit_tiles[(i * 2) + 1] << 8) |
	     (digit_tiles[(i * 2)])) ;
    tile_index++ ;
  }

  /* Chargement des tiles de fond de requester en mmoire. */
  for (i = 0; i < NB_REQUESTER_TILES * (32 / 2); i++) {
    tiles_memory[tile_index] =
      (u16) ((requester_tiles[(i * 2) + 1] << 8) |
	     (requester_tiles[(i * 2)])) ;
    tile_index++ ;
  }

  /* Chargement des tiles de fonte en mmoire. */
  for (i = 0; i < NB_FONT_TILES * (32 / 2); i++) {
    tiles_memory[tile_index] =
      (u16) ((font_tiles[(i * 2) + 1] << 8) |
	     (font_tiles[(i * 2)])) ;
    tile_index++ ;
  }

  /* Chargement de la map de fond et de requester */
  /* et effacement de la map du terrain de jeu.   */
  map_index = 0 ;
  for (j = 0; j < 20; j++) {          /* Boucle verticale. */
    for (i=0; i<30; i++) {        /* Boucle honrizontale. */
      /* Terrain de jeu. */
      background_video_memory[(j * (256 / 8)) + i] =
	(u16) ((background_map[map_index]) |
	       (BACKGROUND_PALETTE_SLOT  << 12)) ;
      /* Effacement du requester. */
      requesters_video_memory[(j * (256 / 8)) + i] =
	(u16) ((REQUESTER_TILES_INDEX + requester_map[map_index]) |
	       (FONT_PALETTE_SLOT  << 12)) ;
      /* Effacement de la map du terrain de jeu. */
      playground_video_memory[(j * (256 / 8)) + i] =
	(u16) (EMPTY_BLOCK_TILE_INDEX | (BLUE_BLOCKS_PALETTE_SLOT  << 12)) ;
      map_index++ ;
    }
  }

  init_palette () ;

  /* Positionnement des offsets de scrolling en haut  gauche (0, 0). */
  REG_BG0HOFS = 0 ;
  REG_BG0VOFS = 0 ;
  REG_BG1HOFS = 0 ;
  REG_BG1VOFS = 0 ;

  set_screen_mode (MODE_0 | BG0_ENABLE | BG1_ENABLE) ;

  print_reserve_icons () ;
  /* Affichage d'un score initialement  0. */
  print_score (0) ;
}
