#include <string.h>
#include <gba.h>
#include <screen_mode.h>
#include <timer.h>
#include "game.h"
#include "control.h"
#include "pause_gfx.h"
#include "fame.h"
#include "persistent.h"
#include "blocks.h"
#include "pause.h"
#include "init.h"
#include "print.h"




void handle_pause_input ()
{
  u8 cursor_selection, old_cursor_selection ;
  
  cursor_selection = SELECTION_CONTINUE ;
  for (;;) {
    old_cursor_selection = cursor_selection ;

    /* Le bouton B fait reprendre par dfaut. */
    KEY_POOL ;
    if (KEY_HIT (BUTTON_B)) break ;

    if (KEY_HIT (BUTTON_A)) {
      switch (cursor_selection) {
      case SELECTION_CONTINUE: return ;	break ;

      case SELECTION_SAVE:
	if (notify_long_message (TRUE, "Really save ?\nA: Yes.\nB: No.")) {
	  save_game (&current_game) ;
	  notify_long_message (FALSE, "Game saved.") ;
	}
	break ;

      case SELECTION_ABANDON:
	current_game.game_running = FALSE ; return ; break ;

      default:
	notify_long_message (FALSE, "handle_pause_input\nASSERT 1") ;
	break ;
      }
    }

    if (KEY_HIT (KEYPAD_UP)) {
      /* Dplacement vers le haut. */
      if (cursor_selection == SELECTION_CONTINUE)
	cursor_selection = SELECTION_ABANDON ;
      else cursor_selection-- ;
    }
    
    if (KEY_HIT (KEYPAD_DOWN)) {
      /* Dplacement vers le bas. */
      cursor_selection = (cursor_selection + 1) % 3 ;
    }

    /* On change la position du curseur seulement s'il a boug. */
    if (cursor_selection != old_cursor_selection) {
      u16 y = PAUSE_OPTIONS_CONTINUE_Y_OFFSET ;
      switch (cursor_selection) {
      case SELECTION_CONTINUE: y = PAUSE_OPTIONS_CONTINUE_Y_OFFSET ; break ;
      case SELECTION_SAVE: y = PAUSE_OPTIONS_SAVE_Y_OFFSET ; break ;
      case SELECTION_ABANDON: y = PAUSE_OPTIONS_ABANDON_Y_OFFSET ; break ;
      default:
	notify_long_message (FALSE, "handle_pause_input\nASSERT 2") ;
	break ;
      }
      cursor_sprite.attribute0 = cursor_sprite.attribute0 & 0xFF00 ;
      cursor_sprite.attribute0 = cursor_sprite.attribute0 | y ;  /* y */
      /* Transfert des infos de sprite dans la mmoire vido-sprite. */
      while (REG_VCOUNT < 160) ;
      memcpy (sprite_video_memory, &cursor_sprite, sizeof (struct sprite)) ;
    }
  }
}





void show_pause ()
{
  u16 *palette_memory = BACKGROUND_PALETTE_ADDR ;
  u16 *pause_video_memory = SCREEN_BASE_BLOCK_11_ADDR ;
  u16 *tiles_memory = CHAR_BASE_BLOCK_2_ADDR ;
  u16 old_screen_mode ;
  u16 tile_index,  map_index ;
  u16 i, j ;

  /* On dsactive le timer en mettant sont bit de cascade. */
  SUSPEND_GAME_TIMER ;

  /* On sauvegarde le mode vido d'origine. */
  old_screen_mode = REG_DISPCNT ;

  tile_index = 0 ;
  /* Chargement des tiles de fond en mmoire. */
  for (i = 0; i < NB_PAUSE_TILES * (32 / 2); i++) {
    tiles_memory[tile_index] =
      (u16) ((pause_tiles[(i * 2) + 1] << 8) |
	     (pause_tiles[(i * 2)])) ;
    tile_index++ ;
  }

  /* Chargement de la map de fond. */
  map_index = 0 ;
  for (j=0; j<20; j++) {          /* Boucle verticale. */
    for (i=0; i<30; i++) {        /* Boucle honrizontale. */
      /* Terrain de jeu. */
      pause_video_memory[(j * (256 / 8)) + i] =
	(u16) ((pause_map[map_index]) |
	       (MISC_REQUESTERS_PALETTE_SLOT  << 12)) ;
      map_index++ ;
    }
  }

  /* Chargement de la palette de fond en slot 0. */
  i = MISC_REQUESTERS_PALETTE_SLOT * 16 ;
  for (j = 0; j < 16; i++, j++)
    palette_memory[i] = pause_palette[j] ;

  /* Positionnement des offsets de scrolling en haut  gauche (0, 0). */
  REG_BG2HOFS = 0 ;
  REG_BG2VOFS = 0 ;

  /* Positionnement initial du sprite de curseur. */
  /* Efface l'ancienne abscisse. */
  cursor_sprite.attribute1 = cursor_sprite.attribute1 & 0xFE00 ;
  cursor_sprite.attribute1 =
    cursor_sprite.attribute1 | PAUSE_OPTIONS_RIGHT_OFFSET ;
  /* Efface l'ancienne ordonne. */
  cursor_sprite.attribute0 = cursor_sprite.attribute0 & 0xFF00 ;
  /* Mme genre de correction pour l'ordonne. */
  cursor_sprite.attribute0 =
    cursor_sprite.attribute0 | PAUSE_OPTIONS_CONTINUE_Y_OFFSET  ;
  /* Transfert des infos de sprite dans la mmoire vido-sprite. */
  while (REG_VCOUNT < 160) ;
  memcpy (sprite_video_memory, &cursor_sprite, sizeof (struct sprite)) ;

  set_screen_mode (MODE_0 | BG2_ENABLE | OBJ_MAP_1D | OBJ_ENABLE) ;

  handle_pause_input () ;
  /* On restaure le mode vido d'origine. */
  set_screen_mode (old_screen_mode) ;

  /* On ractive le timer en supprimant son bit de cascade. */
  RELAUNCH_GAME_TIMER ;
}
