#include <gba.h>
#include "screen_mode.h"
#include "game.h"
#include "print.h"
#include "digits.h"
#include "blocks.h"
#include "char_table.h"
#include "init.h"
#include "control.h"
#include "background.h"




void clear_requester_inside (void)
{
  u8 line, column ;
  u16 blank ;

  blank = (char_to_tile_table[' '] + FONT_TILES_INDEX) |
    (FONT_PALETTE_SLOT << 12) ;
  for (line = 0; line < REQUESTER_NB_LINES; line++) {
    for (column = 0; column < REQUESTER_NB_COLUMNS; column++)
      requesters_video_memory[((line + REQUESTER_TOP_OFFSET) * (256 / 8)) +
			      (column + REQUESTER_LEFT_OFFSET)] = blank ;
  }
}





void do_print_at (u16 line, u16 column, const unsigned char *str)
{
  u8 i ;
  u16 c ;

  for (i = 0; str[i] != '\0'; i++) {
    c = (u16) str[i] ;
    /* Scrolling vertical d au dpassement de la taille horizontale. */
    if (column >= REQUESTER_NB_COLUMNS) {
      column = 0 ;
      line++ ;
    }

    /* Cas du saut de ligne. */
    if (c == '\n') {
      line++ ;
      column = 0 ;
      continue ;
      }

    /* Gestion du scrolling vertical d au */
    /* dpassement de la taille verticale. */
    if (line >= REQUESTER_NB_LINES) {
      KEY_POOL ;
      while (! (KEY_HIT (BUTTON_A))) { KEY_POOL ; }
      clear_requester_inside () ;
      line = 0 ;
      column = 0 ;
    }

    /* Correction pour tomber au bon indice dans la table des tiles. */
    c = char_to_tile_table[c] + FONT_TILES_INDEX ;
    requesters_video_memory
      [((line + REQUESTER_TOP_OFFSET) * (256/8)) +
       (column + REQUESTER_LEFT_OFFSET)] =
      (u16) c | (FONT_PALETTE_SLOT << 12) ;
    column++ ;
  }
}





BOOL notify_long_message (BOOL expect_answer, const unsigned char *str)
{
  u8 line, column ;
  u16 old_screen_mode ;
  BOOL ret_code = FALSE ;

  /* On sauvegarde le mode vido d'origine. */
  old_screen_mode = REG_DISPCNT ;

  /* Effacement de la zone d'affichage. */
  clear_requester_inside () ;

  line = 0 ;
  column = 0 ;
  do_print_at (line, column, str) ;

  /* On autorise l'affichage du layer de requesters */
  /* et on laisse celui du fond.                    */
  set_screen_mode (MODE_0 | BG0_ENABLE | BG3_ENABLE) ;

  if (expect_answer == FALSE) {
    /* On attend une pression sur le bouton A      */
    /* B ou select pour faire disparatre le tout. */
    for (;;) {
      KEY_POOL ;
      if ((KEY_HIT (BUTTON_A)) || (KEY_HIT (BUTTON_B)) ||
	  (KEY_HIT (BUTTON_START))) break ;
      }
    ret_code = FALSE ;
  }
  else {
    for (;;) {
      /* On attend une pression sur le bouton A */
      /* ou B pour faire disparatre le tout.     */
      KEY_POOL ;
      if (KEY_HIT (BUTTON_A)) { ret_code = TRUE ; break ; }
      if (KEY_HIT (BUTTON_B)) { ret_code = FALSE ; break ; }
    }
  }

  /* On restaure le mode vido d'origine. */
  set_screen_mode (old_screen_mode) ;

  return (ret_code) ;
}





void print_score (unsigned int score)
{
  short column ;
  unsigned int div ;
  unsigned int remain = 0 ;
  u16 tile_on_screen ;

  div = score ;
  /* Affichage sur 8 chiffres. */
  for (column = 7; column >= 0; column --) {
    remain = div % 10 ;
    div = div / 10 ;
    tile_on_screen = (SCORE_AREA_Y * (256 / 8)) + (SCORE_AREA_X + column) ;
    /* Premire tile du chiffre. */
    background_video_memory[tile_on_screen] =
      (u16) ((DIGIT_TILES_INDEX + remain) | (BACKGROUND_PALETTE_SLOT << 12)) ;
    /* Seconde tile du chiffre. */
    tile_on_screen = ((SCORE_AREA_Y + 1) * (256 / 8)) +
      (SCORE_AREA_X + column) ;
    background_video_memory[tile_on_screen] =
      (u16) ((DIGIT_TILES_INDEX + remain + (NB_DIGITS_TILES / 2)) |
	     (BACKGROUND_PALETTE_SLOT << 12)) ;
  }
}



void draw_block_tile (u16 tile_info, short x, short y)
{
  u16 tile_on_screen ;

  tile_on_screen =
    (y * (256 / 8)) /* Offset de ligne. */
    /* 256 : Largeur map en pixels. */
    /* 8 : Nbre pixels par tile. */
    + x ;  /* Offset de colonne. */
  playground_video_memory[tile_on_screen] = tile_info ;
  tile_on_screen++ ;
  playground_video_memory[tile_on_screen] = tile_info + 1 ;
  tile_on_screen =
    ((y + 1) * (256 / 8)) /* Offset de ligne. */
    /* 256 : Largeur map en pixels. */
    /* 8 : Nbre pixels par tile. */
    + x ;  /* Offset de colonne. */
  playground_video_memory[tile_on_screen] = tile_info + 2 ;
  tile_on_screen++ ;
  playground_video_memory[tile_on_screen] = tile_info + 3 ;
}



void print_small_2_digits_integer_at (short x, short y, unsigned char nb,
				      unsigned char palette_slot)
{
  u16 tile_on_screen ;
  short column ;
  unsigned int div ;
  unsigned int remain = 0 ;

  div = nb ;
  /* Premire ligne. */
  /* Affichage sur 2 chiffres. */
  for (column = 1; column >= 0; column --) {
    remain = div % 10 ;
    div = div / 10 ;
    tile_on_screen = (y * (256 / 8)) + (x + column) ;
    /* Seule tile du chiffre. */
    playground_video_memory[tile_on_screen] =
      (u16) ((START_BLOCK_COUNT_DIGIT_TILES_INDEX + remain) |
	     (palette_slot << 12)) ;
  }
}




void print_reserve_icons ()
{
  short x, y ;
  u16 tile_info ;

  /* Reprsentation du Deep_Charge. */
  y = BONUS_AREA_Y ;
  x = BONUS_AREA_X ;
  tile_info =
    COLOR_INDEP_DEEPCHARGE_TILE_INDEX |
    (COLOR_INDEP_BLOCKS_PALETTE_SLOT << 12) ;
  draw_block_tile (tile_info, x, y) ;
  /* Reprsentation du Painter. */
  x = BONUS_AREA_X + 2 ;
  tile_info =
    COLOR_INDEP_PAINTER_TILE_INDEX | (COLOR_INDEP_BLOCKS_PALETTE_SLOT << 12) ;
  draw_block_tile (tile_info, x, y) ;
  /* Reprsentation du Line_Col_Cleaner. */
  x = BONUS_AREA_X + 4 ;
  tile_info =
    PLAYER_LINE_COL_CLEANER_UP_TILE_INDEX |
    (COLOR_INDEP_BLOCKS_PALETTE_SLOT << 12) ;
  draw_block_tile (tile_info, x, y) ;
  /* Reprsentation de l'Antidote. */
  x = BONUS_AREA_X + 6 ;
  tile_info =
    COLOR_INDEP_ANTIDOTE_TILE_INDEX | (COLOR_INDEP_BLOCKS_PALETTE_SLOT << 12) ;
  draw_block_tile (tile_info, x, y) ;
  x = BONUS_AREA_X + 8 ;
  tile_info = ERASER_UP_TILE_INDEX | (COLOR_INDEP_BLOCKS_PALETTE_SLOT << 12) ;
  draw_block_tile (tile_info, x, y) ;
}


void print_reserve (unsigned char *reserve)
{
  /* Nombre de Deep_Charge_Red. */
  print_small_2_digits_integer_at
    (BONUS_Deep_Charge_AREA_X,
     BONUS_Deep_Charge_Red_AREA_Y,
     reserve[BONUS_Deep_Charge_Red_INDEX], RED_BLOCKS_PALETTE_SLOT) ;
  /* Nombre de Deep_Charge_Blue. */
  print_small_2_digits_integer_at
    (BONUS_Deep_Charge_AREA_X,
     BONUS_Deep_Charge_Blue_AREA_Y,
     reserve[BONUS_Deep_Charge_Blue_INDEX], BLUE_BLOCKS_PALETTE_SLOT) ;
  /* Nombre de Deep_Charge_Green. */
  print_small_2_digits_integer_at
    (BONUS_Deep_Charge_AREA_X,
     BONUS_Deep_Charge_Green_AREA_Y,
     reserve[BONUS_Deep_Charge_Green_INDEX], GREEN_BLOCKS_PALETTE_SLOT) ;
  /* Nombre de Deep_Charge_Yellow. */
  print_small_2_digits_integer_at
    (BONUS_Deep_Charge_AREA_X,
     BONUS_Deep_Charge_Yellow_AREA_Y,
     reserve[BONUS_Deep_Charge_Yellow_INDEX], YELLOW_BLOCKS_PALETTE_SLOT) ;
  /* Nombre de Painter_Red. */
  print_small_2_digits_integer_at
    (BONUS_Painter_AREA_X,
     BONUS_Painter_Red_AREA_Y,
     reserve[BONUS_Painter_Red_INDEX], RED_BLOCKS_PALETTE_SLOT) ;
  /* Nombre de Painter_Blue. */
  print_small_2_digits_integer_at
    (BONUS_Painter_AREA_X,
     BONUS_Painter_Blue_AREA_Y,
     reserve[BONUS_Painter_Blue_INDEX], BLUE_BLOCKS_PALETTE_SLOT) ;
  /* Nombre de Painter_Green. */
  print_small_2_digits_integer_at
    (BONUS_Painter_AREA_X,
     BONUS_Painter_Green_AREA_Y,
     reserve[BONUS_Painter_Green_INDEX], GREEN_BLOCKS_PALETTE_SLOT) ;
  /* Nombre de Painter_Yellow. */
  print_small_2_digits_integer_at
    (BONUS_Painter_AREA_X,
     BONUS_Painter_Yellow_AREA_Y,
     reserve[BONUS_Painter_Yellow_INDEX], YELLOW_BLOCKS_PALETTE_SLOT) ;
  /* Nombre de Line_Col_Cleaner. */
  print_small_2_digits_integer_at
    (BONUS_Line_Col_Cleaner_AREA_X,
     BONUS_Line_Col_Cleaner_AREA_Y,
     reserve[BONUS_Line_Col_Cleaner_INDEX], COLOR_INDEP_DIGITS_PALETTE_SLOT) ;
  /* Nombre d'Antidote. */
  print_small_2_digits_integer_at
    (BONUS_Antidote_AREA_X,
     BONUS_Antidote_AREA_Y,
     reserve[BONUS_Antidote_INDEX], COLOR_INDEP_DIGITS_PALETTE_SLOT) ;
  /* Nombre d'Eraser . */
  print_small_2_digits_integer_at
    (BONUS_Eraser_AREA_X,
     BONUS_Eraser_AREA_Y,
     reserve[BONUS_Eraser_INDEX], COLOR_INDEP_DIGITS_PALETTE_SLOT) ;
}
