#include <string.h>
#include <gba.h>
#include <screen_mode.h>
#include <interrupt.h>
#include "game.h"
#include "control.h"
#include "print.h"
#include "init.h"
#include "starter.h"
#include "blocks.h"
#include "starter_gfx.h"
#include "pause_gfx.h"
#include "ints.h"
#include "wav_ok.h"



short handle_starter_input ()
{
  u8 cursor_selection, old_cursor_selection ;
  
  cursor_selection = SELECTION_NORMAL ;
  for (;;) {
    old_cursor_selection = cursor_selection ;

    /* Le bouton B ne fait rien par dfaut. */
    /* Les bouton A et START dmarrent le jeu. */
    KEY_POOL ;
    if ((KEY_HIT (BUTTON_A)) || (KEY_HIT (BUTTON_START))) {
      switch (cursor_selection) {
      case SELECTION_NORMAL:
	play_sound (g_wav_ok) ;
	/* En mode de jeu normal, on a PERCENT_CHANCES_OF_BONUS de chances */
	/* d'obtenir un bloc bonus dans le tableau et comme projectile.    */
	return (PERCENT_CHANCES_OF_BONUS) ;
	break ;

      case SELECTION_NOBONUS:
	play_sound (g_wav_ok) ;
	/* En mode sans bonus, on s'arrange pour avoir une proba ngative. */
	return (-1) ;
	break ;

      default:
	notify_long_message (FALSE, "handle_starter_input\nASSERT 1") ;
	break ;
      }
    }

    if (KEY_HIT (KEYPAD_UP)) {
      /* Dplacement vers le haut. */
      if (cursor_selection == SELECTION_NORMAL)
	cursor_selection = SELECTION_NOBONUS ;
      else cursor_selection-- ;
    }
    
    if (KEY_HIT (KEYPAD_DOWN)) {
      /* Dplacement vers le bas. */
      cursor_selection = (cursor_selection + 1) % 2 ;
    }

    /* On change la position du curseur seulement s'il a boug. */
    if (cursor_selection != old_cursor_selection) {
      u16 y = STARTER_OPTIONS_NORMAL_Y_OFFSET ;
      switch (cursor_selection) {
      case SELECTION_NORMAL: y = STARTER_OPTIONS_NORMAL_Y_OFFSET ; break ;
      case SELECTION_NOBONUS: y = STARTER_OPTIONS_NOBONUS_Y_OFFSET ; break ;
      default:
	notify_long_message (FALSE, "handle_starter_input\nASSERT 2") ;
	break ;
      }
      cursor_sprite.attribute0 = cursor_sprite.attribute0 & 0xFF00 ;
      cursor_sprite.attribute0 = cursor_sprite.attribute0 | y ;  /* y */
      /* Transfert des infos de sprite dans la mmoire vido-sprite. */
      while (REG_VCOUNT < 160) ;
      memcpy (sprite_video_memory, &cursor_sprite, sizeof (struct sprite)) ;
    }
  }
}





short starter ()
{
  u16 *palette_memory = BACKGROUND_PALETTE_ADDR ;
  u16 *starter_video_memory = SCREEN_BASE_BLOCK_11_ADDR ;
  u16 *tiles_memory = CHAR_BASE_BLOCK_2_ADDR ;
  u16 old_screen_mode ;
  u16 map_index ;
  u16 i, j ;
  short proba ;

  /* On sauvegarde le mode vido d'origine. */
  old_screen_mode = REG_DISPCNT ;
  set_screen_mode (0) ;

  /* Layer de requesters divers de saisie. */
  set_background2 (BACKGROUND_PRIORITY_1 |
		   BACKGROUND_CHAR_BASE_BLOCK_2 |
		   BACKGROUND_COLOR_16 |
		   BACKGROUND_SCREEN_BASE_BLOCK_11 |
		   TEXT_BACKGROUND_256x256) ;

  /* Chargement des tiles de fond en mmoire. */
  for (i = 0; i < NB_STARTER_TILES * (32 / 2); i++) {
    tiles_memory[i] =
      (u16) ((starter_tiles[(i * 2) + 1] << 8) |
	     (starter_tiles[(i * 2)])) ;
  }

  /* Chargement de la map de fond. */
  map_index = 0 ;
  for (j=0; j<20; j++) {          /* Boucle verticale. */
    for (i=0; i<30; i++) {        /* Boucle honrizontale. */
      /* Terrain de jeu. */
      starter_video_memory[(j * (256 / 8)) + i] =
	(u16) ((starter_map[map_index]) |
	       (MISC_REQUESTERS_PALETTE_SLOT  << 12)) ;
      map_index++ ;
    }
  }

  /* Chargement de la palette de fond en slot 0. */
  i = MISC_REQUESTERS_PALETTE_SLOT * 16 ;
  for (j = 0; j < 16; i++, j++) palette_memory[i] = starter_palette[j] ;

  /* Positionnement des offsets de scrolling en haut  gauche (0, 0). */
  REG_BG2HOFS = 0 ;
  REG_BG2VOFS = 0 ;

  /* Positionnement initial du sprite de curseur. */
  /* Efface l'ancienne abscisse. */
  cursor_sprite.attribute1 = cursor_sprite.attribute1 & 0xFE00 ;
  cursor_sprite.attribute1 =
    cursor_sprite.attribute1 | STARTER_OPTIONS_RIGHT_OFFSET ;
  /* Efface l'ancienne ordonne. */
  cursor_sprite.attribute0 = cursor_sprite.attribute0 & 0xFF00 ;
  /* Mme genre de correction pour l'ordonne. */
  cursor_sprite.attribute0 =
    cursor_sprite.attribute0 | STARTER_OPTIONS_NORMAL_Y_OFFSET  ;
  /* Transfert des infos de sprite dans la mmoire vido-sprite. */
  while (REG_VCOUNT < 160) ;
  memcpy (sprite_video_memory, &cursor_sprite, sizeof (struct sprite)) ;

  set_screen_mode (MODE_0 | BG2_ENABLE | OBJ_MAP_1D | OBJ_ENABLE) ;

  proba = handle_starter_input () ;

  /* On restaure le mode vido d'origine. */
  set_screen_mode (old_screen_mode) ;
  return (proba) ;
}
